@@ -20,33 +20,24 @@
 #define DOWNLOADER_VIEW_H
 
 #include "ephy-dialog.h"
+#include "ephy-download.h"
 
 #include <glib-object.h>
 #include <glib.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+G_BEGIN_DECLS
+
+#define EPHY_TYPE_DOWNLOADER_VIEW		(downloader_view_get_type ())
+#define EPHY_DOWNLOADER_VIEW(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_DOWNLOADER_VIEW, DownloaderView))
+#define EPHY_DOWNLOADER_VIEW_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_DOWNLOADER_VIEW, DownloaderViewClass))
+#define EPHY_IS_DOWNLOADER_VIEW(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_DOWNLOADER_VIEW))
+#define EPHY_IS_DOWNLOADER_VIEW_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_DOWNLOADER_VIEW))
+#define EPHY_DOWNLOADER_VIEW_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_DOWNLOADER_VIEW, DownloaderViewClass))
 
 typedef struct DownloaderView DownloaderView;
 typedef struct DownloaderViewClass DownloaderViewClass;
-
-#define DOWNLOADER_VIEW_TYPE             (downloader_view_get_type ())
-#define DOWNLOADER_VIEW(obj)             (GTK_CHECK_CAST ((obj), DOWNLOADER_VIEW_TYPE, DownloaderView))
-#define DOWNLOADER_VIEW_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), DOWNLOADER_VIEW, DownloaderViewClass))
-#define IS_DOWNLOADER_VIEW(obj)          (GTK_CHECK_TYPE ((obj), DOWNLOADER_VIEW_TYPE))
-#define IS_DOWNLOADER_VIEW_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), DOWNLOADER_VIEW))
-
 typedef struct DownloaderViewPrivate DownloaderViewPrivate;
 
-typedef enum
-{
-	DOWNLOAD_STATUS_DOWNLOADING,
-	DOWNLOAD_STATUS_PAUSED,
-	DOWNLOAD_STATUS_RESUMING,
-	DOWNLOAD_STATUS_COMPLETED
-} DownloadStatus;
-
 struct DownloaderView
 {
         EphyDialog parent;
@@ -56,10 +47,6 @@ struct DownloaderView
 struct DownloaderViewClass
 {
         EphyDialogClass parent_class;
-
-	void (*download_remove)    (DownloaderView *dv, gpointer persist_object);
-	void (*download_pause)     (DownloaderView *dv, gpointer persist_object);
-	void (*download_resume)    (DownloaderView *dv, gpointer persist_object);
 };
 
 GType           downloader_view_get_type              (void);
@@ -67,29 +54,11 @@ GType           downloader_view_get_type              (void);
 DownloaderView *downloader_view_new                   (void);
 
 void            downloader_view_add_download          (DownloaderView *dv,
-						       gchar *filename,
-						       gchar *source,
-						       gchar *dest,
-						       gpointer persist_object);
+						       EphyDownload *download);
 
 void            downloader_view_remove_download       (DownloaderView *dv,
-						       gpointer persist_object);
+						       EphyDownload *download);
 
-void            downloader_view_set_download_status   (DownloaderView *dv,
-						       DownloadStatus status,
-						       gpointer persist_object);
-
-void            downloader_view_set_download_progress (DownloaderView *dv,
-						       glong elapsed,
-						       glong remaining,
-						       gfloat speed,
-						       gint size_total,
-						       gint size_done,
-						       gfloat progress,
-						       gpointer persist_object);
-
-#ifdef __cplusplus
-}
-#endif
+G_END_DECLS
 
 #endif
