@@ -21,25 +21,22 @@
 
 #include "ephy-embed.h"
 #include "ephy-embed-single.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-history.h"
-#include "downloader-view.h"
 
 #include <glib-object.h>
 #include <glib.h>
 
 G_BEGIN_DECLS
 
+#define EPHY_TYPE_EMBED_SHELL		(ephy_embed_shell_get_type ())
+#define EPHY_EMBED_SHELL_IMPL		(ephy_embed_shell_get_impl ())
+#define EPHY_EMBED_SHELL(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShell))
+#define EPHY_EMBED_SHELL_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
+#define EPHY_IS_EMBED_SHELL(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED_SHELL))
+#define EPHY_IS_EMBED_SHELL_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED_SHELL))
+#define EPHY_EMBED_SHELL_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellClass))
+
 typedef struct EphyEmbedShellClass EphyEmbedShellClass;
-
-#define EPHY_EMBED_SHELL_TYPE             (ephy_embed_shell_get_type ())
-#define EPHY_EMBED_SHELL_IMPL             (ephy_embed_shell_get_impl ())
-#define EPHY_EMBED_SHELL(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), EPHY_EMBED_SHELL_TYPE, EphyEmbedShell))
-#define EPHY_EMBED_SHELL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), EPHY_EMBED_SHELL_TYPE, EphyEmbedShellClass))
-#define IS_EPHY_EMBED_SHELL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EPHY_EMBED_SHELL_TYPE))
-#define IS_EPHY_EMBED_SHELL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), EPHY_EMBED_SHELL_TYPE))
-#define EPHY_EMBED_SHELL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), EPHY_EMBED_SHELL_TYPE, EphyEmbedShellClass))
-
 typedef struct EphyEmbedShell EphyEmbedShell;
 typedef struct EphyEmbedShellPrivate EphyEmbedShellPrivate;
 
@@ -55,17 +52,9 @@ struct EphyEmbedShellClass
 {
         GObjectClass parent_class;
 
-	void		(* new_window)          (EphyEmbedShell *shell,
-					         EphyEmbed **new_embed,
-						 EmbedChromeMask chromemask);
-
-	void		(* command)             (EphyEmbedShell *shell,
-						 char *command,
-						 char *param);
-
 	/* Methods */
-	EphyHistory    * (* get_global_history)  (EphyEmbedShell *shell);
-	DownloaderView * (* get_downloader_view) (EphyEmbedShell *shell);
+	EphyHistory * (* get_global_history)  (EphyEmbedShell *shell);
+	GObject     * (* get_downloader_view) (EphyEmbedShell *shell);
 };
 
 GType             ephy_embed_shell_get_type            (void);
@@ -74,21 +63,13 @@ GType             ephy_embed_shell_get_impl            (void);
 
 EphyEmbedShell   *ephy_embed_shell_new                 (const char *type);
 
-EphyFaviconCache *ephy_embed_shell_get_favicon_cache   (EphyEmbedShell *ges);
-
-void              ephy_embed_shell_add_embed           (EphyEmbedShell *ges,
-							EphyEmbed *embed);
-
-void              ephy_embed_shell_remove_embed        (EphyEmbedShell *ges,
-							EphyEmbed *embed);
-
-EphyEmbed        *ephy_embed_shell_get_active_embed    (EphyEmbedShell *ges);
-
-GList            *ephy_embed_shell_get_embeds          (EphyEmbedShell *ges);
+GObject          *ephy_embed_shell_get_favicon_cache   (EphyEmbedShell *ges);
 
 EphyHistory      *ephy_embed_shell_get_global_history  (EphyEmbedShell *shell);
 
-DownloaderView   *ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell);
+GObject          *ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell);
+
+GObject		 *ephy_embed_shell_get_encodings       (EphyEmbedShell *shell);
 
 EphyEmbedSingle  *ephy_embed_shell_get_embed_single    (EphyEmbedShell *shell);
 
