@@ -15,8 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.c,v 1.27 2003/08/03 23:01:35 chpe Exp $
- *
+ *  $Id: ephy-history.c,v 1.34 2003/10/19 13:29:46 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +25,6 @@
 #include "ephy-types.h"
 #include "ephy-history.h"
 #include "ephy-file-helpers.h"
-#include "ephy-autocompletion-source.h"
 #include "ephy-debug.h"
 #include "ephy-node-common.h"
 
@@ -35,13 +33,17 @@
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
+#define EPHY_HISTORY_XML_ROOT	 "ephy_history"
 #define EPHY_HISTORY_XML_VERSION "1.0"
 
 /* how often to save the history, in milliseconds */
 #define HISTORY_SAVE_INTERVAL (60 * 5 * 1000)
 
+/* if you change this remember to change also the user interface description */
 #define HISTORY_PAGE_OBSOLETE_DAYS 10
 
+#define EPHY_HISTORY_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_HISTORY, EphyHistoryPrivate))
+
 struct EphyHistoryPrivate
 {
 	char *xml_file;
@@ -54,6 +56,7 @@ struct EphyHistoryPrivate
 	GHashTable *pages_hash;
 	GStaticRWLock *pages_hash_lock;
 	int autosave_timeout;
+	guint update_hosts_idle;
 };
 
 enum
@@ -71,8 +74,6 @@ static void
 ephy_history_init (EphyHistory *tab);
 static void
 ephy_history_finalize (GObject *object);
-static void
-ephy_history_autocompletion_source_init (EphyAutocompletionSourceIface *iface);
 
 static GObjectClass *parent_class = NULL;
 
@@ -98,86 +99,14 @@ ephy_history_get_type (void)
                         (GInstanceInitFunc) ephy_history_init
                 };
 
-		static const GInterfaceInfo autocompletion_source_info =
-		{
-			(GInterfaceInitFunc) ephy_history_autocompletion_source_init,
-			NULL,
-			NULL
-		};
-
                 ephy_history_type = g_type_register_static (G_TYPE_OBJECT,
 							      "EphyHistory",
 							      &our_info, 0);
-
-		g_type_add_interface_static (ephy_history_type,
-					     EPHY_TYPE_AUTOCOMPLETION_SOURCE,
-					     &autocompletion_source_info);
         }
 
         return ephy_history_type;
 }
 
-static void
-ephy_history_autocompletion_source_set_basic_key (EphyAutocompletionSource *source,
-						  const gchar *basic_key)
-{
-	/* nothing to do here */
-}
-
-static void
-ephy_history_autocompletion_source_foreach (EphyAutocompletionSource *source,
-					    const gchar *current_text,
-					    EphyAutocompletionSourceForeachFunc func,
-					    gpointer data)
-{
-	GPtrArray *children;
-	int i;
-	EphyHistory *eb = EPHY_HISTORY (source);
-	GTime now;
-
-	now = time (NULL);
-
-	children = ephy_node_get_children (eb->priv->pages);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-		const char *url, *title;
-		int last_visit, visits;
-		guint32 score;
-
-		kid = g_ptr_array_index (children, i);
-		g_assert (kid != NULL);
-
-		url = ephy_node_get_property_string
-			(kid, EPHY_NODE_PAGE_PROP_LOCATION);
-		title = ephy_node_get_property_string
-			(kid, EPHY_NODE_PAGE_PROP_TITLE);
-		last_visit = ephy_node_get_property_int
-			(kid, EPHY_NODE_PAGE_PROP_LAST_VISIT);
-		visits = ephy_node_get_property_int
-			(kid, EPHY_NODE_PAGE_PROP_VISITS);
-		score = MAX (visits - ((now - last_visit) >> 15), 1);
-
-		func (source, url,
-		      url, url, FALSE,
-		      FALSE, score, data);
-	}
-	ephy_node_thaw (eb->priv->pages);
-}
-
-static void
-ephy_history_emit_data_changed (EphyHistory *eb)
-{
-	g_signal_emit_by_name (eb, "data-changed");
-}
-
-static void
-ephy_history_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
-{
-	iface->foreach = ephy_history_autocompletion_source_foreach;
-	iface->set_basic_key = ephy_history_autocompletion_source_set_basic_key;
-}
-
 static void
 ephy_history_class_init (EphyHistoryClass *klass)
 {
@@ -197,30 +126,8 @@ ephy_history_class_init (EphyHistoryClass *klass)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
-}
 
-static void
-ephy_history_load (EphyHistory *eb)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-
-	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
-		return;
-
-	doc = xmlParseFile (eb->priv->xml_file);
-	g_return_if_fail (doc != NULL);
-
-	root = xmlDocGetRootElement (doc);
-
-	for (child = root->children; child != NULL; child = child->next)
-	{
-		EphyNode *node;
-
-		node = ephy_node_new_from_xml (eb->priv->db, child);
-	}
-
-	xmlFreeDoc (doc);
+	g_type_class_add_private (object_class, sizeof(EphyHistoryPrivate));
 }
 
 static gboolean
@@ -376,10 +283,77 @@ periodic_save_cb (EphyHistory *eh)
 	return TRUE;
 }
 
-static gboolean
-unref_empty_host (EphyNode *node)
+static void
+update_host_on_child_remove (EphyNode *node)
 {
-	ephy_node_unref (node);
+	GPtrArray *children;
+	int i, host_last_visit, new_host_last_visit = 0;
+
+	host_last_visit = ephy_node_get_property_int
+			(node, EPHY_NODE_PAGE_PROP_LAST_VISIT);
+
+	children = ephy_node_get_children (node);
+	for (i = 0; i < children->len; i++)
+	{
+		EphyNode *kid;
+		int last_visit;
+
+		kid = g_ptr_array_index (children, i);
+
+		last_visit = ephy_node_get_property_int
+                        (kid, EPHY_NODE_PAGE_PROP_LAST_VISIT);
+
+		if (last_visit > new_host_last_visit)
+		{
+			new_host_last_visit = last_visit;
+		}
+	}
+	ephy_node_thaw (node);
+
+	if (host_last_visit != new_host_last_visit)
+	{
+		GValue value = { 0, };
+
+		g_value_init (&value, G_TYPE_INT);
+		g_value_set_int (&value, new_host_last_visit);
+		ephy_node_set_property (node, EPHY_NODE_PAGE_PROP_LAST_VISIT,
+				        &value);
+		g_value_unset (&value);
+	}
+}
+
+static gboolean
+update_hosts (EphyHistory *eh)
+{
+	GPtrArray *children;
+	int i;
+	GList *empty = NULL;
+
+	children = ephy_node_get_children (eh->priv->hosts);
+	for (i = 0; i < children->len; i++)
+	{
+		EphyNode *kid;
+
+		kid = g_ptr_array_index (children, i);
+
+		if (kid != eh->priv->pages)
+		{
+			if (ephy_node_get_n_children (kid) > 0)
+			{
+				update_host_on_child_remove (kid);
+			}
+			else
+			{
+				empty = g_list_prepend (empty, kid);
+			}
+		}
+	}
+	ephy_node_thaw (eh->priv->hosts);
+
+	g_list_foreach (empty, (GFunc)ephy_node_unref, NULL);
+	g_list_free (empty);
+
+	eh->priv->update_hosts_idle = 0;
 
 	return FALSE;
 }
@@ -390,9 +364,10 @@ page_removed_from_host_cb (EphyNode *node,
 		           guint old_index,
 		           EphyHistory *eb)
 {
-	if (ephy_node_get_n_children (node) == 0)
+	if (!eb->priv->update_hosts_idle)
 	{
-		g_idle_add ((GSourceFunc)unref_empty_host, node);
+		eb->priv->update_hosts_idle = g_idle_add
+			((GSourceFunc)update_hosts, eb);
 	}
 }
 
@@ -415,7 +390,8 @@ ephy_history_init (EphyHistory *eb)
 	GValue value = { 0, };
 	EphyNodeDb *db;
 
-        eb->priv = g_new0 (EphyHistoryPrivate, 1);
+        eb->priv = EPHY_HISTORY_GET_PRIVATE (eb);
+	eb->priv->update_hosts_idle = 0;
 
 	db = ephy_node_db_new (EPHY_NODE_DB_HISTORY);
 	eb->priv->db = db;
@@ -475,8 +451,9 @@ ephy_history_init (EphyHistory *eb)
 
 	ephy_node_add_child (eb->priv->hosts, eb->priv->pages);
 
-	ephy_history_load (eb);
-	ephy_history_emit_data_changed (eb);
+	ephy_node_db_load_from_file (eb->priv->db, eb->priv->xml_file,
+				     EPHY_HISTORY_XML_ROOT,
+				     EPHY_HISTORY_XML_VERSION);
 
 	g_hash_table_foreach (eb->priv->hosts_hash,
 			      (GHFunc) connect_page_removed_from_host,
@@ -492,13 +469,12 @@ ephy_history_init (EphyHistory *eb)
 static void
 ephy_history_finalize (GObject *object)
 {
-        EphyHistory *eb;
+        EphyHistory *eb = EPHY_HISTORY (object);
 
-	g_return_if_fail (IS_EPHY_HISTORY (object));
-
-	eb = EPHY_HISTORY (object);
-
-        g_return_if_fail (eb->priv != NULL);
+	if (eb->priv->update_hosts_idle)
+	{
+		g_source_remove (eb->priv->update_hosts_idle);
+	}
 
 	ephy_history_save (eb);
 
@@ -514,8 +490,6 @@ ephy_history_finalize (GObject *object)
 
 	g_source_remove (eb->priv->autosave_timeout);
 
-        g_free (eb->priv);
-
 	LOG ("Global history finalized");
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -524,11 +498,11 @@ ephy_history_finalize (GObject *object)
 EphyHistory *
 ephy_history_new ()
 {
-	EphyHistory *tab;
+	EphyHistory *eh;
 
-	tab = EPHY_HISTORY (g_object_new (EPHY_HISTORY_TYPE, NULL));
+	eh = EPHY_HISTORY (g_object_new (EPHY_TYPE_HISTORY, NULL));
 
-	return tab;
+	return eh;
 }
 
 static void
@@ -724,7 +698,6 @@ ephy_history_visited (EphyHistory *eh, EphyNode *node)
 	eh->priv->last_page = node;
 
 	g_signal_emit (G_OBJECT (eh), ephy_history_signals[VISITED], 0, url);
-	ephy_history_emit_data_changed (eh);
 }
 
 int
