@@ -54,7 +54,7 @@ class GFilePicker : public nsIFilePicker
 		returnCancel = nsIFilePicker::returnCancel,
 		returnReplace = nsIFilePicker::returnReplace };
 
-	GFilePicker(FileFormat *aFileFormats = nsnull);
+	GFilePicker();
 	virtual ~GFilePicker();
 
 	/* additional members */
@@ -62,8 +62,6 @@ class GFilePicker : public nsIFilePicker
 				    const char *aTitle, PRInt16 aMode);
 	NS_METHOD SanityCheck(PRBool *retIsSane);
 
-	PRInt16 mSelectedFileFormat;
-
   private:
 	NS_METHOD HandleFilePickerResult();
 	NS_METHOD ValidateFilePickerResult(PRInt16 *retval);
@@ -81,9 +79,6 @@ class GFilePicker : public nsIFilePicker
 
 	GtkWidget *mParentWidget;	
 	GtkWidget *mFileSelector;
-	GtkWidget *mFormatChooser;
-
-	FileFormat *mFileFormats;
 };
 
 #endif
