@@ -14,10 +14,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: mozilla-embed-persist.cpp,v 1.15 2003/10/19 00:27:47 marco Exp $
  */
 
-#include "ProgressListener.h"
 #include "EphyWrapper.h"
+#include "EphyHeaderSniffer.h"
 #include "mozilla-embed.h"
 #include "mozilla-embed-persist.h"
 
@@ -26,6 +28,8 @@
 #include <nsString.h>
 #include <nsCWebBrowserPersist.h>
 #include <nsNetUtil.h>
+#include <nsIHistoryEntry.h>
+#include <nsISHEntry.h>
 
 static void
 mozilla_embed_persist_class_init (MozillaEmbedPersistClass *klass);
@@ -39,10 +43,12 @@ impl_save (EphyEmbedPersist *persist);
 static gresult 
 impl_cancel (EphyEmbedPersist *persist);
 
+#define MOZILLA_EMBED_PERSIST_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_PERSIST, MozillaEmbedPersistPrivate))
+
 struct MozillaEmbedPersistPrivate
 {
 	nsCOMPtr<nsIWebBrowserPersist> mPersist;
-	GProgressListener *mProgress;
+//	GProgressListener *mProgress;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -68,7 +74,7 @@ mozilla_embed_persist_get_type (void)
                 };
 
                 mozilla_embed_persist_type = 
-				g_type_register_static (EPHY_EMBED_PERSIST_TYPE,
+				g_type_register_static (EPHY_TYPE_EMBED_PERSIST,
                                                         "MozillaEmbedPersist",
                                                         &our_info, (GTypeFlags)0);
         }
@@ -80,41 +86,33 @@ static void
 mozilla_embed_persist_class_init (MozillaEmbedPersistClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	EphyEmbedPersistClass *persist_class;
+	EphyEmbedPersistClass *persist_class = EPHY_EMBED_PERSIST_CLASS (klass);
 	
         parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
-	persist_class = EPHY_EMBED_PERSIST_CLASS (klass);
 	
         object_class->finalize = mozilla_embed_persist_finalize;
 
 	persist_class->save = impl_save;
 	persist_class->cancel = impl_cancel;
+
+	g_type_class_add_private (object_class, sizeof(MozillaEmbedPersistPrivate));
 }
 
 static void
 mozilla_embed_persist_init (MozillaEmbedPersist *persist)
 {
-        persist->priv = g_new0 (MozillaEmbedPersistPrivate, 1);
+        persist->priv = MOZILLA_EMBED_PERSIST_GET_PRIVATE (persist);
+
       	persist->priv->mPersist = do_CreateInstance (NS_WEBBROWSERPERSIST_CONTRACTID);
 }
 
 static void
 mozilla_embed_persist_finalize (GObject *object)
 {
-        MozillaEmbedPersist *persist;
+        MozillaEmbedPersist *persist = MOZILLA_EMBED_PERSIST (object);
 
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_MOZILLA_EMBED_PERSIST (object));
-
-        persist = MOZILLA_EMBED_PERSIST (object);
-
-	persist->priv->mPersist->SetProgressListener (nsnull);
 	persist->priv->mPersist = nsnull;
-	
-        g_return_if_fail (persist->priv != NULL);
 
-	g_free (persist->priv);
-	
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -149,15 +147,15 @@ static gresult
 impl_save (EphyEmbedPersist *persist)
 {
 	nsresult rv;
-	nsAutoString s;
 	char *filename;
 	char *uri;
 	int max_size;
 	EphyEmbed *embed;
 	EmbedPersistFlags flags;
-	EphyWrapper *wrapper = NULL;
 	PRUint32 persistFlags = 0;
 
+	/* FIXME implement max size */
+
 	g_object_ref (persist);
 	
 	g_object_get (persist,
@@ -169,77 +167,72 @@ impl_save (EphyEmbedPersist *persist)
 		      NULL);
 	
 	g_return_val_if_fail (filename != NULL, G_FAILED);
-	
-	nsCOMPtr<nsIWebBrowserPersist> bpersist =
-		MOZILLA_EMBED_PERSIST (persist)->priv->mPersist;
-	if (!bpersist) return G_FAILED;
 
-	nsCOMPtr<nsIURI> linkURI;
-	linkURI = nsnull;
-	if (uri)
-	{
-        	s.AssignWithConversion(uri);
-	      	rv = NS_NewURI(getter_AddRefs(linkURI), s);
-      		if (NS_FAILED(rv) || !linkURI) return G_FAILED;
-	}
-
-        nsCOMPtr<nsILocalFile> file;
-       	rv = NS_NewLocalFile(NS_ConvertUTF8toUCS2(filename), PR_TRUE, getter_AddRefs(file)); 
-        if (NS_FAILED(rv) || !file) return G_FAILED;
-
-	nsCOMPtr<nsILocalFile> path;
-        if (flags & EMBED_PERSIST_SAVE_CONTENT)
-	{
-		char *datapath;
-		datapath = g_strconcat (filename, ".content", NULL);
-                NS_NewLocalFile(NS_ConvertUTF8toUCS2(datapath), PR_TRUE, getter_AddRefs(path));
-		g_free (datapath);
-	}
-        else
-	{
-                path = nsnull;
-	}
-	
-	nsCOMPtr<nsIDOMWindow> parent;
-	parent = nsnull;
-	
+	EphyWrapper *wrapper = NULL;
 	if (embed)
 	{
 	        wrapper = (EphyWrapper *) mozilla_embed_get_ephy_wrapper (MOZILLA_EMBED(embed));
-		g_return_val_if_fail (wrapper != NULL, G_FAILED);	
-		wrapper->GetDOMWindow (getter_AddRefs (parent));
-	}
-
-	persistFlags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
-
-	size_t len = strlen(filename);
-	if((filename[len-1] == 'z' && filename[len-2] == 'g') ||
-	   (filename[len-1] == 'Z' && filename[len-2] == 'G'))
-	{
-                persistFlags |= nsIWebBrowserPersist::PERSIST_FLAGS_NO_CONVERSION;
-	}
-
-	if (flags & EMBED_PERSIST_BYPASSCACHE)
-	{
-		persistFlags |= nsIWebBrowserPersist::PERSIST_FLAGS_BYPASS_CACHE;
-	}
-
-	if (flags & EMBED_PERSIST_FROMCACHE)
-	{
-		persistFlags |= nsIWebBrowserPersist::PERSIST_FLAGS_FROM_CACHE;
-	}
-
-	bpersist->SetPersistFlags (persistFlags);
-
-	GProgressListener *aProgress = new GProgressListener ();
-	MOZILLA_EMBED_PERSIST (persist)->priv->mProgress = aProgress;
-
-	if (uri == NULL)
-	{
 		g_return_val_if_fail (wrapper != NULL, G_FAILED);
-		
-		nsCOMPtr<nsIDOMDocument> DOMDocument;
+	}
 
+	/* we must have one of uri or wrapper */
+	g_assert (wrapper != NULL || uri != NULL);
+
+	/* Get a temp filename to save to */
+	nsCOMPtr<nsIProperties> dirService(do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv));
+	if (!dirService) return G_FAILED;
+	nsCOMPtr<nsIFile> tmpFile;
+	dirService->Get("TmpD", NS_GET_IID(nsIFile), getter_AddRefs(tmpFile));
+	static short unsigned int tmpRandom = 0;
+	nsAutoString tmpNo;
+	tmpNo.AppendInt(tmpRandom++);
+	nsAutoString saveFile(NS_LITERAL_STRING("-sav"));
+	saveFile += tmpNo;
+	saveFile += NS_LITERAL_STRING("tmp");
+	tmpFile->Append(saveFile);
+
+	/* Get the uri to save to */
+	nsCOMPtr<nsIURI> inURI;
+	nsCAutoString sURI;
+	if (uri)
+	{
+		sURI.Assign (uri);
+	}
+	else
+	{
+		rv = wrapper->GetDocumentUrl (sURI);
+		if (NS_FAILED(rv)) return G_FAILED;
+	}
+      	rv = NS_NewURI(getter_AddRefs(inURI), sURI);
+	if (NS_FAILED(rv) || !inURI) return G_FAILED;
+
+	/* Filename to save to */
+	nsAutoString inFilename;
+	inFilename.AssignWithConversion (filename);
+
+	/* Get post data */
+	nsCOMPtr<nsIInputStream> postData;
+	if (wrapper)
+	{
+		PRInt32 sindex;
+
+		nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(wrapper->mWebBrowser));
+		nsCOMPtr<nsISHistory> sessionHistory;
+		webNav->GetSessionHistory(getter_AddRefs(sessionHistory));
+		nsCOMPtr<nsIHistoryEntry> entry;
+		sessionHistory->GetIndex(&sindex);
+		sessionHistory->GetEntryAtIndex(sindex, PR_FALSE, getter_AddRefs(entry));
+		nsCOMPtr<nsISHEntry> shEntry(do_QueryInterface(entry));
+		if (shEntry)
+		{
+			shEntry->GetPostData(getter_AddRefs(postData));
+		}
+	}
+
+	/* Get the DOM document if a uri is not specified */
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	if (!uri)
+	{		
 		if (flags & EMBED_PERSIST_MAINDOC)
 		{
                 	rv = wrapper->GetMainDOMDocument (getter_AddRefs(DOMDocument));
@@ -249,46 +242,22 @@ impl_save (EphyEmbedPersist *persist)
                 	rv = wrapper->GetDOMDocument (getter_AddRefs(DOMDocument));
 		}
         	if (NS_FAILED(rv) || !DOMDocument) return G_FAILED;
-
-		nsCOMPtr<nsIDocument> document =
-                                do_QueryInterface (DOMDocument, &rv);
-        	if (NS_FAILED(rv) || !document) return G_FAILED;
-
-#if MOZILLA_SNAPSHOT > 13
-		nsIURI *uri;
-		uri = document->GetDocumentURI ();
-		if (!uri) return G_FAILED;
-#elif MOZILLA_SNAPSHOT > 11
-		nsIURI *uri;
-		uri = document->GetDocumentURL ();
-		if (!uri) return G_FAILED;
-#else
-	        nsCOMPtr<nsIURI> uri;
-        	rv = document->GetDocumentURL (getter_AddRefs(uri));
-        	if (NS_FAILED(rv) || !uri) return G_FAILED;
-#endif
-
-        	aProgress->InitForPersist (bpersist, parent,
-                   	                   uri, file, 
-                           	           ACTION_OBJECT_NOTIFY,
-					   persist,
-					   (flags & EMBED_PERSIST_SHOW_PROGRESS));
-
-		rv = bpersist->SaveDocument (DOMDocument, file, path, nsnull, 0, 0);
-		if (NS_FAILED(rv)) return G_FAILED;
 	}
-	else
-	{
-        	aProgress->InitForPersist (bpersist, parent,
-                 	                   linkURI, file,
-                         	           ACTION_OBJECT_NOTIFY,
-                                 	   persist, 
-					   (flags & EMBED_PERSIST_SHOW_PROGRESS));
 
-		rv = bpersist->SaveURI (linkURI, nsnull, nsnull, nsnull, nsnull, file);
-		if (NS_FAILED(rv)) return G_FAILED;
-	}
-	
+	/* Create an header sniffer and do the save */
+	nsCOMPtr<nsIWebBrowserPersist> webPersist =
+		MOZILLA_EMBED_PERSIST (persist)->priv->mPersist;
+	if (!webPersist) return G_FAILED;
+
+	EphyHeaderSniffer* sniffer = new EphyHeaderSniffer
+		(webPersist, MOZILLA_EMBED_PERSIST (persist),
+		 tmpFile, inURI, DOMDocument, postData,
+		 inFilename, flags & EMBED_PERSIST_BYPASSCACHE);
+	if (!sniffer) return G_FAILED;
+ 
+	webPersist->SetProgressListener(sniffer);
+	rv = webPersist->SaveURI(inURI, nsnull, nsnull, nsnull, nsnull, tmpFile);
+	if (NS_FAILED (rv)) return G_FAILED;
+
 	return G_OK;
 }
-
