@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: mozilla-embed-single.cpp,v 1.45 2003/10/13 17:38:14 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -21,7 +23,6 @@
 #endif
 
 #include "glib.h"
-#include "ephy-string.h"
 #include "ephy-debug.h"
 #include "gtkmozembed.h"
 #include "mozilla-embed-single.h"
@@ -36,7 +37,7 @@
 
 #include <time.h>
 #include <libgnome/gnome-i18n.h>
-#include <string.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsIPrefService.h>
@@ -55,131 +56,14 @@
 #include <nsCCookieManager.h>
 #include <nsCPasswordManager.h>
 
+// FIXME: For setting the locale. hopefully gtkmozembed will do itself soon
+#include <nsIChromeRegistry.h>
+#include <nsILocaleService.h>
+
 #define MOZILLA_PROFILE_DIR  "/mozilla"
 #define MOZILLA_PROFILE_NAME "epiphany"
 #define MOZILLA_PROFILE_FILE "prefs.js"
-
-/* language groups names */
-static const
-struct
-{
-	gchar *title;
-	LanguageGroup group;
-}
-lang_groups[] =
-{
-        { N_("_Arabic"),		LG_ARABIC },
-        { N_("_Baltic"),		LG_BALTIC },
-        { N_("Central _European"),	LG_CENTRAL_EUROPEAN },
-        { N_("Chi_nese"),		LG_CHINESE },
-        { N_("_Cyrillic"),		LG_CYRILLIC },
-        { N_("_Greek"),			LG_GREEK },
-        { N_("_Hebrew"),		LG_HEBREW },
-        { N_("_Indian"),		LG_INDIAN },
-        { N_("_Japanese"),		LG_JAPANESE },
-        { N_("_Korean"),		LG_KOREAN },
-        { N_("_Turkish"),		LG_TURKISH },
-        { N_("_Unicode"),		LG_UNICODE },
-        { N_("_Vietnamese"),		LG_VIETNAMESE },
-        { N_("_Western"),		LG_WESTERN },
-        { N_("_Other"),			LG_OTHER }
-};
-static const guint n_lang_groups = G_N_ELEMENTS (lang_groups);
-
-/**
- * translatable encodings titles
- * NOTE: if you add /change encodings, please also update the schema file
- * epiphany.schemas.in
- */
-static const 
-struct
-{
-	gchar *title;
-	gchar *name;
-	LanguageGroup group;
-}
-encodings[] =
-{ 
-	/* translators: access keys need only be unique within the same LG_group */
-	{ N_("Arabic (_IBM-864)"),                  "IBM864",                LG_ARABIC },
-	{ N_("Arabic (ISO-_8859-6)"),               "ISO-8859-6",            LG_ARABIC },
-	{ N_("Arabic (_MacArabic)"),                "x-mac-arabic",          LG_ARABIC },
-	{ N_("Arabic (_Windows-1256)"),             "windows-1256",          LG_ARABIC },
-	{ N_("Baltic (_ISO-8859-13)"),              "ISO-8859-13",           LG_BALTIC },
-	{ N_("Baltic (I_SO-8859-4)"),               "ISO-8859-4",            LG_BALTIC },
-	{ N_("Baltic (_Windows-1257)"),             "windows-1257",          LG_BALTIC },
-	{ N_("Central European (_IBM-852)"),        "IBM852",                LG_CENTRAL_EUROPEAN },
-	{ N_("Central European (I_SO-8859-2)"),     "ISO-8859-2",	     LG_CENTRAL_EUROPEAN },
-	{ N_("Central European (_MacCE)"),          "x-mac-ce",              LG_CENTRAL_EUROPEAN },
-	{ N_("Central European (_Windows-1250)"),   "windows-1250",          LG_CENTRAL_EUROPEAN },
-	{ N_("Croatian (Mac_Croatian)"),            "x-mac-croatian",        LG_CENTRAL_EUROPEAN },
-	{ N_("Chinese Simplified (_GB18030)"),      "gb18030",               LG_CHINESE },
-	{ N_("Chinese Simplified (G_B2312)"),       "GB2312",                LG_CHINESE },
-	{ N_("Chinese Simplified (GB_K)"),          "x-gbk",                 LG_CHINESE },
-	{ N_("Chinese Simplified (_HZ)"),           "HZ-GB-2312",	     LG_CHINESE },
-	{ N_("Chinese Simplified (_ISO-2022-CN)"),  "ISO-2022-CN",           LG_CHINESE },
-	{ N_("Chinese Traditional (Big_5)"),        "Big5",                  LG_CHINESE },
-	{ N_("Chinese Traditional (Big5-HK_SCS)"),  "Big5-HKSCS",	     LG_CHINESE },
-	{ N_("Chinese Traditional (_EUC-TW)"),      "x-euc-tw",              LG_CHINESE },
-	{ N_("Cyrillic (_IBM-855)"),                "IBM855",                LG_CYRILLIC },
-	{ N_("Cyrillic (I_SO-8859-5)"),             "ISO-8859-5",	     LG_CYRILLIC },
-	{ N_("Cyrillic (IS_O-IR-111)"),             "ISO-IR-111",	     LG_CYRILLIC },
-	{ N_("Cyrillic (_KOI8-R)"),                 "KOI8-R",                LG_CYRILLIC },
-	{ N_("Cyrillic (_MacCyrillic)"),            "x-mac-cyrillic",        LG_CYRILLIC },
-	{ N_("Cyrillic (_Windows-1251)"),           "windows-1251",          LG_CYRILLIC },
-	{ N_("Cyrillic/Russian (_CP-866)"),         "IBM866",                LG_CYRILLIC },
-	{ N_("Cyrillic/Ukrainian (_KOI8-U)"),       "KOI8-U",                LG_CYRILLIC },
-	{ N_("Cyrillic/Ukrainian (Mac_Ukrainian)"), "x-mac-ukrainian",       LG_CYRILLIC },
-	{ N_("Greek (_ISO-8859-7)"),                "ISO-8859-7",            LG_GREEK },
-	{ N_("Greek (_MacGreek)"),                  "x-mac-greek",           LG_GREEK },
-	{ N_("Greek (_Windows-1253)"),              "windows-1253",          LG_GREEK },
-	{ N_("Gujarati (_MacGujarati)"),            "x-mac-gujarati",        LG_INDIAN },
-	{ N_("Gurmukhi (Mac_Gurmukhi)"),            "x-mac-gurmukhi",        LG_INDIAN },
-	{ N_("Hindi (Mac_Devanagari)"),             "x-mac-devanagari",      LG_INDIAN },
-	{ N_("Hebrew (_IBM-862)"),                  "IBM862",                LG_HEBREW },
-	{ N_("Hebrew (IS_O-8859-8-I)"),             "ISO-8859-8-I",          LG_HEBREW },
-	{ N_("Hebrew (_MacHebrew)"),                "x-mac-hebrew",          LG_HEBREW },
-	{ N_("Hebrew (_Windows-1255)"),             "windows-1255",          LG_HEBREW },
-	{ N_("_Visual Hebrew (ISO-8859-8)"),        "ISO-8859-8",            LG_HEBREW },
-	{ N_("Japanese (_EUC-JP)"),                 "EUC-JP",                LG_JAPANESE },
-	{ N_("Japanese (_ISO-2022-JP)"),            "ISO-2022-JP",           LG_JAPANESE },
-	{ N_("Japanese (_Shift-JIS)"),              "Shift_JIS",             LG_JAPANESE },
-	{ N_("Korean (_EUC-KR)"),                   "EUC-KR",                LG_KOREAN },
-	{ N_("Korean (_ISO-2022-KR)"),              "ISO-2022-KR",           LG_KOREAN },
-	{ N_("Korean (_JOHAB)"),                    "x-johab",               LG_KOREAN },
-	{ N_("Korean (_UHC)"),                      "x-windows-949",         LG_KOREAN },
-	{ N_("Turkish (_IBM-857)"),                 "IBM857",                LG_TURKISH },
-	{ N_("Turkish (I_SO-8859-9)"),              "ISO-8859-9",            LG_TURKISH },
-	{ N_("Turkish (_MacTurkish)"),              "x-mac-turkish",         LG_TURKISH },
-	{ N_("Turkish (_Windows-1254)"),            "windows-1254",          LG_TURKISH },
-	{ N_("Unicode (UTF-_7)"),                   "UTF-7",                 LG_UNICODE },
-	{ N_("Unicode (UTF-_8)"),                   "UTF-8",                 LG_UNICODE },
-	{ N_("Vietnamese (_TCVN)"),                 "x-viet-tcvn5712",       LG_VIETNAMESE },
-	{ N_("Vietnamese (_VISCII)"),               "VISCII",                LG_VIETNAMESE },
-	{ N_("Vietnamese (V_PS)"),                  "x-viet-vps",            LG_VIETNAMESE },
-	{ N_("Vietnamese (_Windows-1258)"),         "windows-1258",          LG_VIETNAMESE },
-	{ N_("Western (_IBM-850)"),                 "IBM850",                LG_WESTERN },
-	{ N_("Western (I_SO-8859-1)"),              "ISO-8859-1",            LG_WESTERN },
-	{ N_("Western (IS_O-8859-15)"),             "ISO-8859-15",           LG_WESTERN },
-	{ N_("Western (_MacRoman)"),                "x-mac-roman",           LG_WESTERN },
-	{ N_("Western (_Windows-1252)"),            "windows-1252",          LG_WESTERN },
-	{ N_("_Armenian (ARMSCII-8)"),              "armscii-8",             LG_OTHER },
-	{ N_("_Celtic (ISO-8859-14)"),              "ISO-8859-14",           LG_OTHER },
-	{ N_("_Farsi (MacFarsi)"),                  "x-mac-farsi",           LG_OTHER },
-	{ N_("_Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER },
-	{ N_("_Icelandic (MacIcelandic)"),          "x-mac-icelandic",       LG_OTHER },
-	{ N_("_Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER },
-	{ N_("_Romanian (MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
-	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
-	{ N_("South _European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
-	{ N_("Thai (TIS-_620)"),                    "TIS-620",               LG_OTHER },
-#if MOZILLA_SNAPSHOT >= 9
-	{ N_("Thai (IS_O-8859-11)"),                "iso-8859-11",           LG_OTHER },
-	{ N_("_Thai (Windows-874)"),                "windows-874",           LG_OTHER },
-#endif	
-	{ N_("_User Defined"),                      "x-user-defined",        LG_OTHER },
-};
-static const guint n_encodings = G_N_ELEMENTS (encodings);
+#define DEFAULT_PROFILE_FILE SHARE_DIR"/default-prefs.js"
 
 static void
 mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass);
@@ -196,20 +80,10 @@ impl_set_offline_mode (EphyEmbedSingle *shell,
 static gresult           
 impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 			  const char* url);
-static gresult           
-impl_get_encodings (EphyEmbedSingle *shell,
-		    LanguageGroup group,
-		    gboolean elide_underscores,
-		    GList **encodings_list);
-static gresult           
-impl_get_language_groups (EphyEmbedSingle *shell,
-		          GList **groups);
 static gresult
 impl_get_font_list (EphyEmbedSingle *shell,
 		    const char *langGroup,
-		    const char *fontType,
-		    GList **fontList,
-		    char **default_font);
+		    GList **fontList);
 static gresult           
 impl_list_cookies (EphyEmbedSingle *shell,
 		   GList **cookies);
@@ -240,6 +114,8 @@ static void mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 					              guint chrome_mask,
                            		              EphyEmbedSingle *shell);
 
+#define MOZILLA_EMBED_SINGLE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED_SINGLE, MozillaEmbedSinglePrivate))
+
 struct MozillaEmbedSinglePrivate
 {
 	char *user_prefs;
@@ -270,7 +146,7 @@ mozilla_embed_single_get_type (void)
                         (GInstanceInitFunc) mozilla_embed_single_init
                 };
 
-                mozilla_embed_single_type = g_type_register_static (EPHY_EMBED_SINGLE_TYPE,
+                mozilla_embed_single_type = g_type_register_static (EPHY_TYPE_EMBED_SINGLE,
 								   "MozillaEmbedSingle",
 								   &our_info, (GTypeFlags)0);
         }
@@ -282,30 +158,29 @@ static void
 mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	EphyEmbedSingleClass *shell_class;
+	EphyEmbedSingleClass *shell_class = EPHY_EMBED_SINGLE_CLASS (klass);
 	
 	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
-	shell_class = EPHY_EMBED_SINGLE_CLASS (klass);
 	
         object_class->finalize = mozilla_embed_single_finalize;
 
 	shell_class->clear_cache = impl_clear_cache;
 	shell_class->set_offline_mode = impl_set_offline_mode;
 	shell_class->load_proxy_autoconf = impl_load_proxy_autoconf;
-	shell_class->get_encodings = impl_get_encodings;
-	shell_class->get_language_groups = impl_get_language_groups;
 	shell_class->get_font_list = impl_get_font_list;
 	shell_class->list_cookies = impl_list_cookies;
 	shell_class->remove_cookies = impl_remove_cookies;
 	shell_class->list_passwords = impl_list_passwords;
 	shell_class->remove_passwords = impl_remove_passwords;
 	shell_class->show_file_picker = impl_show_file_picker;
+
+	g_type_class_add_private (object_class, sizeof(MozillaEmbedSinglePrivate));
 }
 
 EphyEmbedSingle *
 mozilla_embed_single_new (void)
 {
-	return EPHY_EMBED_SINGLE (g_object_new (MOZILLA_EMBED_SINGLE_TYPE, NULL));
+	return EPHY_EMBED_SINGLE (g_object_new (MOZILLA_TYPE_EMBED_SINGLE, NULL));
 }
 
 static gboolean
@@ -316,71 +191,34 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
         prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
 	if (prefService == NULL) return FALSE;
 
+        /* read our predefined default prefs */
+        nsresult rv;
+        nsCOMPtr<nsILocalFile> file;
+        rv = NS_NewNativeLocalFile(
+                NS_LITERAL_CSTRING(DEFAULT_PROFILE_FILE),
+                PR_TRUE, getter_AddRefs(file));
+        if (NS_FAILED (rv)) return FALSE;
+
+        rv = prefService->ReadUserPrefs (file);                                                                              
+        if (NS_FAILED(rv))
+        {
+                g_warning ("failed to read default preferences, error: %x", rv);
+		return FALSE;
+        }
+                                                                                                                             
+        /* Load the default user preferences as well.  This also makes the
+           prefs to be saved in the user's prefs.js file, instead of messing up
+           our global defaults file. */
+        rv = prefService->ReadUserPrefs (nsnull);
+        if (NS_FAILED(rv))
+        {
+                g_warning ("failed to read user preferences, error: %x", rv);
+        }
+
         nsCOMPtr<nsIPrefBranch> pref;
         prefService->GetBranch ("", getter_AddRefs(pref));
 	if (pref == NULL) return FALSE;
 
-	/* Don't allow mozilla to raise window when setting focus (work around bugs) */
-	pref->SetBoolPref ("mozilla.widget.raise-on-setfocus", PR_FALSE);
-
-	/* set default search engine */
-	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?q=");
-	pref->SetBoolPref ("keyword.enabled", PR_TRUE);
-	pref->SetBoolPref ("security.checkloaduri", PR_FALSE);
-
-	/* dont allow xpi installs from epiphany, there are crashes */
-	pref->SetBoolPref ("xpinstall.enabled", PR_FALSE);
-
-	/* deactivate mailcap and mime.types support */
-	pref->SetCharPref ("helpers.global_mailcap_file", "");
-	pref->SetCharPref ("helpers.global_mime_types_file", "");
-	pref->SetCharPref ("helpers.private_mailcap_file", "");
-	pref->SetCharPref ("helpers.private_mime_types_file", "");
-
-	/* disable sucky XUL ftp view, have nice ns4-like html page instead */
-	pref->SetBoolPref ("network.dir.generate_html", PR_TRUE);
-
-	/* disable usless security warnings */
-	pref->SetBoolPref ("security.warn_entering_secure", PR_FALSE);
-	pref->SetBoolPref ("security.warn_leaving_secure", PR_FALSE);
-	pref->SetBoolPref ("security.warn_submit_insecure", PR_FALSE);
-
-	/* FIXME: We should change this back to true when mozilla bugs
-         * 207000 and 207001 are fixed.
-	 */
-	/* Make sure this is off, we monitor for theme changes and set colors. */
-	pref->SetBoolPref ("browser.display.use_system_colors", PR_FALSE);
-
-	/* Smooth scrolling on */
-	pref->SetBoolPref ("general.smoothScroll", PR_TRUE);
-
-	/* Disable blinking text and marquee, its non-standard and annoying */
-	pref->SetBoolPref ("browser.blink_allowed", PR_FALSE);
-	pref->SetBoolPref ("browser.display.enable_marquee", PR_FALSE);
-
-	/* FIXME: We disable this for now, because enough people complained
-	 * and there are some moz bugs. We either need a user visible pref
-	 * or need to revert to enabling by default for a11y
-         */
-	/* Enable Browsing with the Caret */
-	/*pref->SetBoolPref ("accessibility.browsewithcaret", PR_TRUE);*/
-
-	/* Don't Fetch the Sidebar whats related information, since we don't use it */
-	pref->SetBoolPref ("browser.related.enabled", PR_FALSE);
-
-	/* Line Wrap View->Source */
-	pref->SetBoolPref ("view_source.wrap_long_lines", PR_TRUE);
-
-	/* CTRL-Mousewheel scrolls by one page */
-	pref->SetIntPref ("mousewheel.withcontrolkey.action", 1);
-	pref->SetIntPref ("mousewheel.withcontrolkey.numlines", 1);
-	pref->SetBoolPref ("mousewheel.withcontrolkey.sysnumlines", PR_FALSE);
-
-	/* Enable Image Auto-Resizing */
-	pref->SetBoolPref ("browser.enable_automatic_image_resizing", PR_TRUE);
-
-	/* User agent */
-
 	/* FIXME We need to do this because mozilla doesnt set product
 	sub for embedding apps */
 	pref->SetCharPref ("general.useragent.vendor", "Epiphany");
@@ -533,7 +371,7 @@ mozilla_register_external_protocols (void)
 static void
 mozilla_embed_single_init (MozillaEmbedSingle *mes)
 {
- 	mes->priv = g_new0 (MozillaEmbedSinglePrivate, 1);
+ 	mes->priv = MOZILLA_EMBED_SINGLE_GET_PRIVATE (mes);
 
 	mes->priv->theme_window = NULL;
 	mes->priv->user_prefs =
@@ -544,6 +382,59 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 				  NULL);
 }
 
+static nsresult
+getUILang (nsAString& aUILang)
+{
+	nsresult result;
+
+	nsCOMPtr<nsILocaleService> localeService = do_GetService (NS_LOCALESERVICE_CONTRACTID, &result);
+	if (NS_FAILED (result) || !localeService)
+	{
+		g_warning ("Could not get locale service!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	nsXPIDLString uiLang;
+	result = localeService->GetLocaleComponentForUserAgent (getter_Copies(uiLang));
+	aUILang = uiLang;
+	if (NS_FAILED (result))
+	{
+		g_warning ("Could not determine locale!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	return NS_OK;
+}
+
+static nsresult
+mozilla_init_chrome (void)
+{
+	nsresult result;
+	nsAutoString uiLang;
+
+	nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService (NS_CHROMEREGISTRY_CONTRACTID, &result);
+	if (NS_FAILED (result) || !chromeRegistry)
+	{
+		g_warning ("Could not get the chrome registry!\n");
+		return NS_ERROR_FAILURE;
+	}
+
+	// Set skin to 'classic' so we get native scrollbars.
+	result = chromeRegistry->SelectSkin (NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	// set locale
+	chromeRegistry->SetRuntimeProvider(PR_TRUE);
+
+	result = getUILang(uiLang);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	result = chromeRegistry->SelectLocale (NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
 gboolean
 mozilla_embed_single_init_services (MozillaEmbedSingle *single)
 {
@@ -554,6 +445,9 @@ mozilla_embed_single_init_services (MozillaEmbedSingle *single)
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
 
+	/* Until gtkmozembed does this itself */
+	mozilla_init_chrome ();
+
 	mozilla_init_single (single);
 
 	if (!mozilla_set_default_prefs (single))
@@ -625,18 +519,11 @@ mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 static void
 mozilla_embed_single_finalize (GObject *object)
 {
-	MozillaEmbedSingle *mes;
+	MozillaEmbedSingle *mes = MOZILLA_EMBED_SINGLE (object);
 
 	/* Destroy EphyEmbedSingle before because some
 	 * services depend on xpcom */
-        G_OBJECT_CLASS (parent_class)->finalize (object);
-
-        g_return_if_fail (object != NULL);
-        g_return_if_fail (IS_MOZILLA_EMBED_SINGLE (object));
-
-        mes = MOZILLA_EMBED_SINGLE (object);
-
-        g_return_if_fail (mes->priv != NULL);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 
 	mozilla_notifiers_free ();
 
@@ -648,8 +535,6 @@ mozilla_embed_single_finalize (GObject *object)
 	{
 		gtk_widget_destroy (mes->priv->theme_window);
 	}
-	
-        g_free (mes->priv);
 }
 
 static gresult      
@@ -699,100 +584,10 @@ impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 	return G_OK;
 }
 
-static gint
-encoding_info_cmp (const EncodingInfo *i1, const EncodingInfo *i2)
-{
-	return strcmp (i1->key, i2->key);
-}
-
-static gresult           
-impl_get_encodings (EphyEmbedSingle *shell,
-		   LanguageGroup group,
-		   gboolean elide_underscores,
-		   GList **encodings_list)
-{
-	GList *l = NULL;
-	guint i;
-
-	for (i = 0; i < n_encodings; i++)
-	{
-		if (group == LG_ALL || group == encodings[i].group)
-		{
-			EncodingInfo *info;
-			gchar *elided = NULL;
-
-			info = g_new0 (EncodingInfo, 1);
-
-			info->encoding = g_strdup (encodings[i].name);
-			
-			elided = ephy_string_elide_underscores (_(encodings[i].title));
-
-			if (elide_underscores)
-			{
-				info->title = g_strdup (elided);
-			}
-			else
-			{
-				info->title = g_strdup (_(encodings[i].title));
-			}
-
-			/* collate without underscores */
-			info->key = g_utf8_collate_key (elided, -1);
-
-			info->group = encodings[i].group;
-
-			l = g_list_prepend (l, info);
-			g_free (elided);
-		}
-	}
-
-	*encodings_list = g_list_sort (l, (GCompareFunc) encoding_info_cmp);
-
-	return G_OK;
-}
-
-static gint
-language_group_info_cmp (const LanguageGroupInfo *i1, const LanguageGroupInfo *i2)
-{
-	return strcmp (i1->key, i2->key);
-}
-
-static gresult           
-impl_get_language_groups (EphyEmbedSingle *shell,
-		          GList **groups)
-{
-	GList *l = NULL;
-	guint i;
-	
-	for (i = 0; i < n_lang_groups; i++)
-	{
-		LanguageGroupInfo *info;
-		gchar *elided = NULL;
-		
-		info = g_new0 (LanguageGroupInfo, 1);
-		
-		info->title = g_strdup (_(lang_groups[i].title));
-		info->group = lang_groups[i].group;
-		
-		/* collate without underscores */
-		elided = ephy_string_elide_underscores (info->title);
-		info->key = g_utf8_collate_key (elided, -1);
-		g_free (elided);
-
-		l = g_list_prepend (l, info);
-	}
-	
-	*groups = g_list_sort (l, (GCompareFunc) language_group_info_cmp);
-	
-	return G_OK;
-}
-
 static gresult
 impl_get_font_list (EphyEmbedSingle *shell,
 		    const char *langGroup,
-		    const char *fontType,
-		    GList **fontList,
-		    char **default_font)
+		    GList **fontList)
 {
 	nsresult rv;
 	PRUint32 fontCount;
@@ -807,7 +602,7 @@ impl_get_font_list (EphyEmbedSingle *shell,
 					        &fontCount, &fontArray);
 	if (NS_FAILED (rv)) return G_FAILED;
 
-	for (int i = 0; i < fontCount; i++)
+	for (PRUint32 i = 0; i < fontCount; i++)
 	{
 		char *gFontString;
 
@@ -820,11 +615,6 @@ impl_get_font_list (EphyEmbedSingle *shell,
 
 	*fontList = g_list_reverse (l);
 
-	if (default_font != NULL)
-	{
-		*default_font = g_strdup (fontType);
-	}
-
 	return G_OK;
 }
 
@@ -1001,20 +791,22 @@ impl_show_file_picker (EphyEmbedSingle *shell,
                        FileFormat *file_formats, 
 		       int *ret_file_format)
 {
-	gchar *expanded_directory;
+	char *expanded_directory = NULL;
 	gresult result;
 
-        GFilePicker *filePicker = new GFilePicker (file_formats);
+        GFilePicker *filePicker = new GFilePicker ();
 
-	/* FIXME sane path: expand tilde ... */
-        expanded_directory = g_strdup (directory);
+	if (directory != NULL)
+	{
+	        expanded_directory = gnome_vfs_expand_initial_tilde (directory);
+	}
 
         /* make sure the directory exists, and use the home directory
          * otherwise */
         if (!expanded_directory ||
             !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
         {
-                if (expanded_directory) g_free (expanded_directory);
+                g_free (expanded_directory);
                 expanded_directory = g_strdup (g_get_home_dir());
         }
 
@@ -1033,11 +825,6 @@ impl_show_file_picker (EphyEmbedSingle *shell,
         PRInt16 retval;
         filePicker->Show (&retval);
 
-        if (ret_file_format != NULL)
-        {
-                *ret_file_format = filePicker->mSelectedFileFormat;
-        }
-
         nsCOMPtr<nsILocalFile> local_file;
 	filePicker->GetFile (getter_AddRefs(local_file));
 	nsCAutoString tempFullPathStr;
