@@ -18,11 +18,20 @@
 
 #include "egg-editable-toolbar.h"
 #include "egg-toolbars-model.h"
-#include "eggtoolbar.h"
-#include "eggtoolitem.h"
-#include "eggseparatortoolitem.h"
-#include "eggintl.h"
 
+#include <gtk/gtkvseparator.h>
+#include <gtk/gtkiconfactory.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkdnd.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtktoolbar.h>
+#include <gtk/gtktoolitem.h>
+#include <gtk/gtkseparatortoolitem.h>
+#include <bonobo/bonobo-i18n.h>
 #include <string.h>
 
 static void egg_editable_toolbar_class_init	(EggEditableToolbarClass *klass);
@@ -60,10 +69,11 @@ static GObjectClass *parent_class = NULL;
 
 struct EggEditableToolbarPrivate
 {
-  EggMenuMerge *merge;
+  GtkUIManager *merge;
   EggToolbarsModel *model;
   gboolean edit_mode;
   GtkWidget *selected_toolbar;
+  GtkWidget *dragged_item;
 };
 
 GType
@@ -133,21 +143,23 @@ get_toolbar_nth (EggEditableToolbar *etoolbar,
   return result;
 }
 
-static EggAction *
+static GtkAction *
 find_action (EggEditableToolbar *t,
 	     const char         *name)
 {
-  GList *l = t->priv->merge->action_groups;
-  EggAction *action = NULL;
+  GList *l;
+  GtkAction *action = NULL;
+
+  l = gtk_ui_manager_get_action_groups (t->priv->merge);
 
   g_return_val_if_fail (IS_EGG_EDITABLE_TOOLBAR (t), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
   for (; l != NULL; l = l->next)
     {
-      EggAction *tmp;
+      GtkAction *tmp;
 
-      tmp = egg_action_group_get_action (EGG_ACTION_GROUP (l->data), name);
+      tmp = gtk_action_group_get_action (GTK_ACTION_GROUP (l->data), name);
       if (tmp)
 	action = tmp;
     }
@@ -155,39 +167,6 @@ find_action (EggEditableToolbar *t,
   return action;
 }
 
-static void
-drag_data_received_cb (GtkWidget          *widget,
-		       GdkDragContext     *context,
-		       gint                x,
-		       gint                y,
-		       GtkSelectionData   *selection_data,
-		       guint               info,
-		       guint               time_,
-		       EggEditableToolbar *etoolbar)
-{
-  int pos, toolbar_pos;
-
-  g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (etoolbar));
-
-  pos = egg_toolbar_get_drop_index (EGG_TOOLBAR (widget), x, y);
-  toolbar_pos = get_toolbar_position (etoolbar, widget);
-
-  if (strcmp (selection_data->data, "separator") == 0)
-    {
-      egg_toolbars_model_add_separator (etoolbar->priv->model,
-					toolbar_pos, pos);
-    }
-  else
-    {
-      GdkAtom target;
-
-      target = gtk_drag_dest_find_target (widget, context, NULL);
-      egg_toolbars_model_add_item (etoolbar->priv->model,
-			           toolbar_pos, pos, target,
-			           selection_data->data);
-    }
-}
-
 static void
 drag_data_delete_cb (GtkWidget          *widget,
 		     GdkDragContext     *context,
@@ -197,14 +176,30 @@ drag_data_delete_cb (GtkWidget          *widget,
 
   g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (etoolbar));
 
-  pos = egg_toolbar_get_item_index (EGG_TOOLBAR (widget->parent),
-				    EGG_TOOL_ITEM (widget));
+  pos = gtk_toolbar_get_item_index (GTK_TOOLBAR (widget->parent),
+				    GTK_TOOL_ITEM (widget));
   toolbar_pos = get_toolbar_position (etoolbar, widget->parent);
 
   egg_toolbars_model_remove_item (etoolbar->priv->model,
 			          toolbar_pos, pos);
 }
 
+static void
+drag_begin_cb (GtkWidget          *widget,
+	       GdkDragContext     *context,
+	       EggEditableToolbar *etoolbar)
+{
+	gtk_widget_hide (widget);
+}
+
+static void
+drag_end_cb (GtkWidget          *widget,
+	     GdkDragContext     *context,
+	     EggEditableToolbar *etoolbar)
+{
+	gtk_widget_show (widget);
+}
+
 static void
 drag_data_get_cb (GtkWidget          *widget,
 		  GdkDragContext     *context,
@@ -213,16 +208,16 @@ drag_data_get_cb (GtkWidget          *widget,
 		  guint32             time,
 		  EggEditableToolbar *etoolbar)
 {
-  EggAction *action;
+  GtkAction *action;
   const char *target;
 
   g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (etoolbar));
 
-  action = EGG_ACTION (g_object_get_data (G_OBJECT (widget), "egg-action"));
+  action = GTK_ACTION (g_object_get_data (G_OBJECT (widget), "gtk-action"));
 
   if (action)
     {
-      target = action->name;
+      target = gtk_action_get_name (action);
     }
   else
     {
@@ -233,6 +228,197 @@ drag_data_get_cb (GtkWidget          *widget,
 			  selection_data->target, 8, target, strlen (target));
 }
 
+static void
+set_drag_cursor (GtkWidget *widget)
+{
+  if (widget->window)
+    {
+      GdkCursor *cursor;
+      GdkPixbuf *pixbuf;
+
+      pixbuf = gdk_pixbuf_new_from_file (CURSOR_DIR "/art/hand-open.png", NULL);
+      cursor = gdk_cursor_new_from_pixbuf (gdk_display_get_default (),
+					   pixbuf, 0, 0);
+      gdk_window_set_cursor (widget->window, cursor);
+      gdk_cursor_unref (cursor);
+      g_object_unref (pixbuf);
+    }
+}
+
+static void
+unset_drag_cursor (GtkWidget *widget)
+{
+  if (widget->window)
+    {
+      gdk_window_set_cursor (widget->window, NULL);
+    }
+}
+
+static void
+set_item_drag_source (GtkWidget *item,
+		      GtkAction *action,
+		      gboolean   is_separator)
+{
+  gtk_drag_source_set (item, GDK_BUTTON1_MASK,
+		       source_drag_types, n_source_drag_types,
+		       GDK_ACTION_MOVE);
+
+  if (is_separator)
+    {
+      GtkWidget *icon;
+      GdkPixbuf *pixbuf;
+
+      icon = _egg_editable_toolbar_new_separator_image ();
+      pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (icon));
+      gtk_drag_source_set_icon_pixbuf (item, pixbuf);
+    }
+  else
+    {
+      const char *stock_id;
+      GValue value = { 0, };
+                                                                                                                             
+      g_value_init (&value, G_TYPE_STRING);
+      g_object_get_property (G_OBJECT (action), "stock_id", &value);
+                                                                                                                             
+      stock_id = g_value_get_string (&value);
+      gtk_drag_source_set_icon_stock
+	(item, stock_id ? stock_id : GTK_STOCK_DND);
+      g_value_unset (&value);
+    }
+}
+
+static GtkWidget *
+create_item_from_action (EggEditableToolbar *t,
+			 const char *action_name,
+			 gboolean is_separator)
+{
+  GtkWidget *item;
+  GtkAction *action;
+
+  if (is_separator)
+    {
+      item = GTK_WIDGET (gtk_separator_tool_item_new ());
+      action = NULL;
+    }
+  else
+    {
+      g_signal_emit (G_OBJECT (t), egg_editable_toolbar_signals[ACTION_REQUEST],
+		     0, action_name);
+      action = find_action (t, action_name);
+      item = gtk_action_create_tool_item (action);
+    }
+
+  gtk_widget_show (item);
+
+  g_signal_connect (item, "drag_begin",
+		    G_CALLBACK (drag_begin_cb), t);
+  g_signal_connect (item, "drag_end",
+		    G_CALLBACK (drag_end_cb), t);
+  g_signal_connect (item, "drag_data_get",
+		    G_CALLBACK (drag_data_get_cb), t);
+  g_signal_connect (item, "drag_data_delete",
+		    G_CALLBACK (drag_data_delete_cb), t);
+
+  if (t->priv->edit_mode)
+    {
+      set_drag_cursor (item);
+      gtk_widget_set_sensitive (item, TRUE);
+      set_item_drag_source (item, action, is_separator);
+      gtk_tool_item_set_use_drag_window (GTK_TOOL_ITEM (item), TRUE);
+    }
+
+  return item;
+}
+
+static GtkWidget *
+create_item (EggEditableToolbar *t,
+	     EggToolbarsModel   *model,
+	     int                 toolbar_position,
+	     int                 position)
+{
+  const char *action_name;
+  gboolean is_separator;
+
+  action_name = egg_toolbars_model_item_nth
+		(model, toolbar_position, position,
+		 &is_separator);
+  return create_item_from_action (t, action_name, is_separator);
+}
+
+static gboolean
+data_is_separator (const char *data)
+{
+  return strcmp (data, "separator") == 0;
+}
+
+static void
+set_status_pending (GdkDragContext *context,
+                    GdkDragAction   suggested_action)
+{
+  g_object_set_data (G_OBJECT (context),
+                     "gtk-calendar-status-pending",
+                     GINT_TO_POINTER (suggested_action));
+}
+                                                                                                                             
+static GdkDragAction
+get_status_pending (GdkDragContext *context)
+{
+  return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (context),
+                                             "gtk-calendar-status-pending"));
+}
+
+static void
+drag_data_received_cb (GtkWidget          *widget,
+		       GdkDragContext     *context,
+		       gint                x,
+		       gint                y,
+		       GtkSelectionData   *selection_data,
+		       guint               info,
+		       guint               time,
+		       EggEditableToolbar *etoolbar)
+{
+  GdkDragAction suggested_action;
+  int pos, toolbar_pos;
+
+  suggested_action = get_status_pending (context);
+
+  if (suggested_action)
+    {
+      set_status_pending (context, 0);
+      etoolbar->priv->dragged_item =
+        create_item_from_action (etoolbar, selection_data->data,
+				 data_is_separator (selection_data->data));
+      gdk_drag_status (context, suggested_action, time);
+      return;
+    }
+
+   pos = gtk_toolbar_get_drop_index (GTK_TOOLBAR (widget), x, y);
+   toolbar_pos = get_toolbar_position (etoolbar, widget);
+
+   if (data_is_separator (selection_data->data))
+     {
+       egg_toolbars_model_add_separator (etoolbar->priv->model,
+	  				 toolbar_pos, pos);
+     }
+     else
+     {
+       GdkAtom target;
+       char *type;
+       char *id;
+
+       target = gtk_drag_dest_find_target (widget, context, NULL);
+       type = egg_toolbars_model_get_item_type (etoolbar->priv->model, target);
+       id = egg_toolbars_model_get_item_id (etoolbar->priv->model, type,
+					    selection_data->data);
+       egg_toolbars_model_add_item (etoolbar->priv->model,
+			            toolbar_pos, pos, id, type);
+       g_free (type);
+       g_free (id);
+    }
+
+    gtk_drag_finish (context, TRUE, context->action == GDK_ACTION_MOVE, time);
+}
+
 static void
 remove_toolbar_cb (GtkWidget          *menuitem,
 		   EggEditableToolbar *etoolbar)
@@ -245,6 +431,9 @@ remove_toolbar_cb (GtkWidget          *menuitem,
 
 static void
 popup_toolbar_context_menu_cb (GtkWidget          *toolbar,
+			       gint		   x,
+			       gint		   y,
+			       gint                button_number,
 			       EggEditableToolbar *t)
 {
   GtkWidget *menu;
@@ -282,17 +471,99 @@ popup_toolbar_context_menu_cb (GtkWidget          *toolbar,
     }
 }
 
+static void
+free_dragged_item (EggEditableToolbar *etoolbar)
+{
+  if (etoolbar->priv->dragged_item)
+  {	
+    gtk_widget_destroy (etoolbar->priv->dragged_item);
+    etoolbar->priv->dragged_item = NULL;
+  }
+}
+
+static gboolean
+toolbar_drag_drop_cb (GtkWidget          *widget,
+		      GdkDragContext     *context,
+		      gint                x,
+		      gint                y,
+		      guint               time,
+		      EggEditableToolbar *etoolbar)
+{
+  GdkAtom target;
+
+  free_dragged_item (etoolbar);
+  
+  target = gtk_drag_dest_find_target (widget, context, NULL);
+  if (target != GDK_NONE)
+    {
+      gtk_drag_get_data (widget, context,
+                         target,
+                         time);
+      return TRUE;
+    }
+  
+  return FALSE;
+}
+
+static gboolean
+toolbar_drag_motion_cb (GtkWidget          *widget,
+		        GdkDragContext     *context,
+		        gint                x,
+		        gint                y,
+		        guint               time,
+		        EggEditableToolbar *etoolbar)
+{
+  GdkAtom target;
+  GtkRequisition req;
+
+  target = gtk_drag_dest_find_target (widget, context, NULL);
+  if (target == GDK_NONE)
+    {
+      gdk_drag_status (context, 0, time);
+    }
+  else
+    {
+      set_status_pending (context, context->suggested_action);
+      gtk_drag_get_data (widget, context, target, time);
+    }
+
+  gtk_widget_size_request (etoolbar->priv->dragged_item, &req);
+  gtk_toolbar_highlight_drop_location (GTK_TOOLBAR (widget), x, y,
+				       req.width, req.height);
+
+  return TRUE;
+}
+
+static void
+toolbar_drag_leave_cb (GtkToolbar         *toolbar,
+		       GdkDragContext     *context,
+		       guint               time,
+		       EggEditableToolbar *etoolbar)
+{
+  free_dragged_item (etoolbar);
+  gtk_toolbar_unhighlight_drop_location (toolbar);
+}
+
 static GtkWidget *
 create_toolbar (EggEditableToolbar *t)
 {
   GtkWidget *toolbar;
 
-  toolbar = egg_toolbar_new ();
-  egg_toolbar_set_show_arrow (EGG_TOOLBAR (toolbar), TRUE);
+  toolbar = gtk_toolbar_new ();
+  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), TRUE);
+
   gtk_widget_show (toolbar);
-  gtk_drag_dest_set (toolbar, GTK_DEST_DEFAULT_DROP,
+  gtk_drag_dest_set (toolbar, 0,
 		     dest_drag_types, n_dest_drag_types,
 		     GDK_ACTION_MOVE | GDK_ACTION_COPY);
+ 
+  g_signal_connect (toolbar, "drag_drop",
+		    G_CALLBACK (toolbar_drag_drop_cb), t); 
+  g_signal_connect (toolbar, "drag_motion",
+		    G_CALLBACK (toolbar_drag_motion_cb), t);
+  g_signal_connect (toolbar, "drag_leave",
+		    G_CALLBACK (toolbar_drag_leave_cb), t);
+
   g_signal_connect (toolbar, "drag_data_received",
 		    G_CALLBACK (drag_data_received_cb), t);
   g_signal_connect (toolbar, "popup_context_menu",
@@ -301,74 +572,6 @@ create_toolbar (EggEditableToolbar *t)
   return toolbar;
 }
 
-static void
-set_item_drag_source (GtkWidget *item,
-		      EggAction *action,
-		      gboolean   is_separator)
-{
-  gtk_drag_source_set (item, GDK_BUTTON1_MASK,
-		       source_drag_types, n_source_drag_types,
-		       GDK_ACTION_MOVE);
-
-  if (is_separator)
-    {
-      GtkWidget *icon;
-      GdkPixbuf *pixbuf;
-
-      icon = _egg_editable_toolbar_new_separator_image ();
-      pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (icon));
-      gtk_drag_source_set_icon_pixbuf (item, pixbuf);
-    }
-  else
-    {
-      gtk_drag_source_set_icon_stock
-	(item, action->stock_id ? action->stock_id : GTK_STOCK_DND);
-    }
-}
-
-static GtkWidget *
-create_item (EggEditableToolbar *t,
-	     EggToolbarsModel   *model,
-	     int                 toolbar_position,
-	     int                 position)
-{
-  GtkWidget *item;
-  EggAction *action;
-  const char *action_name;
-  gboolean is_separator;
-
-  action_name = egg_toolbars_model_item_nth
-		(model, toolbar_position, position,
-		 &is_separator);
-  g_signal_emit (G_OBJECT (t), egg_editable_toolbar_signals[ACTION_REQUEST],
-		 0, action_name);
-  action = find_action (t, action_name);
-
-  if (is_separator)
-    {
-      item = GTK_WIDGET (egg_separator_tool_item_new ());
-    }
-  else
-    {
-      item = egg_action_create_tool_item (action);
-      gtk_widget_set_sensitive (item, TRUE);
-    }
-
-  gtk_widget_show (item);
-  g_signal_connect (item, "drag_data_get",
-		    G_CALLBACK (drag_data_get_cb), t);
-  g_signal_connect (item, "drag_data_delete",
-		    G_CALLBACK (drag_data_delete_cb), t);
-
-  if (t->priv->edit_mode)
-    {
-      set_item_drag_source (item, action, is_separator);
-      egg_tool_item_set_use_drag_window (EGG_TOOL_ITEM (item), TRUE);
-    }
-
-  return item;
-}
-
 static void
 toolbar_changed_cb (EggToolbarsModel   *model,
 	            int                 position,
@@ -382,11 +585,11 @@ toolbar_changed_cb (EggToolbarsModel   *model,
 
   if (flags & EGG_TB_MODEL_ICONS_ONLY)
   {
-    egg_toolbar_set_style (EGG_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
   }
   else
   {
-    egg_toolbar_unset_style (EGG_TOOLBAR (toolbar));
+    gtk_toolbar_unset_style (GTK_TOOLBAR (toolbar));
   }
 }
 
@@ -427,8 +630,8 @@ item_added_cb (EggToolbarsModel   *model,
   toolbar = get_toolbar_nth (t, toolbar_position);
   gtk_widget_set_size_request (toolbar, -1, -1);
   item = create_item (t, model, toolbar_position, position);
-  egg_toolbar_insert (EGG_TOOLBAR (toolbar),
-		      EGG_TOOL_ITEM (item), position);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+		      GTK_TOOL_ITEM (item), position);
 }
 
 static void
@@ -441,8 +644,9 @@ item_removed_cb (EggToolbarsModel   *model,
   GtkWidget *item;
 
   toolbar = get_toolbar_nth (t, toolbar_position);
-  item = GTK_WIDGET (egg_toolbar_get_nth_item
-	(EGG_TOOLBAR (toolbar), position));
+  item = GTK_WIDGET (gtk_toolbar_get_nth_item
+	(GTK_TOOLBAR (toolbar), position));
+  g_return_if_fail (item != NULL);
   gtk_container_remove (GTK_CONTAINER (toolbar), item);
 
   if (egg_toolbars_model_n_items (model, toolbar_position) == 0)
@@ -453,24 +657,18 @@ item_removed_cb (EggToolbarsModel   *model,
 }
 
 static void
-egg_editable_toolbar_set_model (EggEditableToolbar *t,
-				EggToolbarsModel   *model)
+toolbars_clean (EggEditableToolbar *t)
 {
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (model));
-  g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (t));
+  GList *children, *l;
 
-  t->priv->model = model;
+  children = gtk_container_get_children (GTK_CONTAINER (t));
 
-  g_signal_connect_object (model, "item_added",
-			   G_CALLBACK (item_added_cb), t, 0);
-  g_signal_connect_object (model, "item_removed",
-			   G_CALLBACK (item_removed_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_added",
-			   G_CALLBACK (toolbar_added_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_removed",
-			   G_CALLBACK (toolbar_removed_cb), t, 0);
-  g_signal_connect_object (model, "toolbar_changed",
-			   G_CALLBACK (toolbar_changed_cb), t, 0);
+  for (l = children; l != NULL; l = l->next)
+    {
+      gtk_widget_destroy (GTK_WIDGET (l->data));
+    }
+
+  g_list_free (children);
 }
 
 static void
@@ -486,18 +684,25 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
   for (i = 0; i < n_toolbars; i++)
     {
       GtkWidget *toolbar;
+      EggTbModelFlags flags;
 
       toolbar = create_toolbar (t);
       gtk_box_pack_start (GTK_BOX (t), toolbar, FALSE, FALSE, 0);
 
+      flags = egg_toolbars_model_get_flags (model, i);
+      if (flags & EGG_TB_MODEL_ICONS_ONLY)
+        {
+          gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+        }
+
       n_items = egg_toolbars_model_n_items (model, i);
       for (l = 0; l < n_items; l++)
         {
           GtkWidget *item;
 
           item = create_item (t, model, i, l);
-	  egg_toolbar_insert (EGG_TOOLBAR (toolbar),
-			      EGG_TOOL_ITEM (item), l);
+	  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+			      GTK_TOOL_ITEM (item), l);
         }
 
       if (n_items == 0)
@@ -508,15 +713,47 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
 }
 
 static void
-egg_editable_toolbar_set_merge (EggEditableToolbar *t,
-				EggMenuMerge       *merge)
+egg_editable_toolbar_set_model (EggEditableToolbar *t,
+				EggToolbarsModel   *model)
 {
-  g_return_if_fail (EGG_IS_MENU_MERGE (merge));
+  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (model));
   g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (t));
 
-  t->priv->merge = merge;
+  if (t->priv->model == model) return;
 
+  t->priv->model = model;
+
+  toolbars_clean (t);
   egg_editable_toolbar_construct (t);
+
+  g_signal_connect_object (model, "item_added",
+			   G_CALLBACK (item_added_cb), t, 0);
+  g_signal_connect_object (model, "item_removed",
+			   G_CALLBACK (item_removed_cb), t, 0);
+  g_signal_connect_object (model, "toolbar_added",
+			   G_CALLBACK (toolbar_added_cb), t, 0);
+  g_signal_connect_object (model, "toolbar_removed",
+			   G_CALLBACK (toolbar_removed_cb), t, 0);
+  g_signal_connect_object (model, "toolbar_changed",
+			   G_CALLBACK (toolbar_changed_cb), t, 0);
+}
+
+static void
+egg_editable_toolbar_set_merge (EggEditableToolbar *t,
+				GtkUIManager       *merge)
+{
+  g_return_if_fail (GTK_IS_UI_MANAGER (merge));
+  g_return_if_fail (IS_EGG_EDITABLE_TOOLBAR (t));
+
+  if (t->priv->merge != merge)
+    {
+      t->priv->merge = merge;
+
+      if (t->priv->model != NULL)
+        {
+          egg_editable_toolbar_construct (t);
+        }
+    }
 }
 
 static void
@@ -581,7 +818,7 @@ egg_editable_toolbar_class_init (EggEditableToolbarClass *klass)
 				   g_param_spec_object ("MenuMerge",
 							"MenuMerge",
 							"Menu merge",
-							EGG_TYPE_MENU_MERGE,
+							GTK_TYPE_UI_MANAGER,
 							G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
 				   PROP_TOOLBARS_MODEL,
@@ -615,7 +852,7 @@ egg_editable_toolbar_finalize (GObject *object)
 }
 
 GtkWidget *
-egg_editable_toolbar_new (EggMenuMerge     *merge,
+egg_editable_toolbar_new (GtkUIManager     *merge,
 			  EggToolbarsModel *model)
 {
   EggEditableToolbar *t;
@@ -643,29 +880,31 @@ egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
       GtkWidget *toolbar;
 
       toolbar = get_toolbar_nth (etoolbar, i);
-      n_items = egg_toolbar_get_n_items (EGG_TOOLBAR (toolbar));
+      n_items = gtk_toolbar_get_n_items (GTK_TOOLBAR (toolbar));
       for (l = 0; l < n_items; l++)
         {
-	  EggToolItem *item;
+	  GtkToolItem *item;
 	  const char *action_name;
           gboolean is_separator;
-	  EggAction *action;
+	  GtkAction *action;
 
           action_name = egg_toolbars_model_item_nth
 		(etoolbar->priv->model, i, l,
 		 &is_separator);
 	  action = find_action (etoolbar, action_name);
 
-	  item = egg_toolbar_get_nth_item (EGG_TOOLBAR (toolbar), l);
-	  egg_tool_item_set_use_drag_window (item, mode);
+	  item = gtk_toolbar_get_nth_item (GTK_TOOLBAR (toolbar), l);
+	  gtk_tool_item_set_use_drag_window (item, mode);
 
           if (mode)
 	    {
+              set_drag_cursor (GTK_WIDGET (item));
 	      gtk_widget_set_sensitive (GTK_WIDGET (item), TRUE);
               set_item_drag_source (GTK_WIDGET (item), action, is_separator);
             }
 	  else
             {
+              unset_drag_cursor (GTK_WIDGET (item));
               gtk_drag_source_unset (GTK_WIDGET (item));
 
               if (!is_separator)
@@ -743,7 +982,7 @@ egg_editable_toolbar_set_drag_dest (EggEditableToolbar   *etoolbar,
         GtkWidget *widget = get_toolbar_nth (etoolbar, i);
 
         gtk_drag_dest_unset (widget);
-        gtk_drag_dest_set (widget, GTK_DEST_DEFAULT_DROP,
+        gtk_drag_dest_set (widget, 0,
                            targets, n_targets,
                            GDK_ACTION_MOVE | GDK_ACTION_COPY);
       }
@@ -874,7 +1113,7 @@ style_set_cb (GtkWidget *widget,
 }
 
 GtkWidget *
-_egg_editable_toolbar_new_separator_image ()
+_egg_editable_toolbar_new_separator_image (void)
 {
   GtkWidget *image = gtk_image_new ();
   update_separator_image (GTK_IMAGE (image));
