@@ -25,15 +25,14 @@
 
 G_BEGIN_DECLS
 
+#define EPHY_TYPE_DIALOG	 (ephy_dialog_get_type ())
+#define EPHY_DIALOG(o)		 (G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_DIALOG, EphyDialog))
+#define EPHY_DIALOG_CLASS(k)	 (G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_DIALOG, EphyDialogClass))
+#define EPHY_IS_DIALOG(o)	 (G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_DIALOG))
+#define EPHY_IS_DIALOG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_DIALOG))
+#define EPHY_DIALOG_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_DIALOG, EphyDialogClass))
+
 typedef struct EphyDialogClass EphyDialogClass;
-
-#define EPHY_DIALOG_TYPE             (ephy_dialog_get_type ())
-#define EPHY_DIALOG(obj)             (GTK_CHECK_CAST ((obj), EPHY_DIALOG_TYPE, EphyDialog))
-#define EPHY_DIALOG_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), EPHY_DIALOG_TYPE, EphyDialogClass))
-#define IS_EPHY_DIALOG(obj)          (GTK_CHECK_TYPE ((obj), EPHY_DIALOG_TYPE))
-#define IS_EPHY_DIALOG_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), EPHY_DIALOG))
-#define EPHY_DIALOG_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), EPHY_DIALOG_TYPE, EphyDialogClass))
-
 typedef struct EphyDialog EphyDialog;
 typedef struct EphyDialogPrivate EphyDialogPrivate;
 
@@ -71,7 +70,6 @@ struct EphyDialogClass
 					 const EphyDialogProperty *properties,
 					 const char *file,
 			                 const char *name);
-	void	    (* destruct)	(EphyDialog *dialog);
 	gint        (* run)		(EphyDialog *dialog);
 	void        (* show)		(EphyDialog *dialog);
 	GtkWidget * (* get_control)     (EphyDialog *dialog,
@@ -92,13 +90,15 @@ void	      ephy_dialog_construct		(EphyDialog *dialog,
 						 const char *file,
 						 const char *name);
 
-void	      ephy_dialog_destruct		(EphyDialog *dialog);
-
 void	      ephy_dialog_add_enum		(EphyDialog *dialog,
 						 int id,
 						 guint n_items,
 						 const char **items);
 
+void	      ephy_dialog_set_size_group	(EphyDialog *dialog,
+						 int *controls_id,
+						 guint n_controls);
+
 gint          ephy_dialog_run			(EphyDialog *dialog);
 
 void          ephy_dialog_show			(EphyDialog *dialog);
