@@ -27,7 +27,7 @@
 
 static void
 add_topics_list (EphyNode *topics, EphyNode *bmk,
-		 xmlNodePtr parent, xmlNsPtr dc_ns, xmlDocPtr doc)
+		 xmlNodePtr parent, xmlNsPtr dc_ns)
 {
 	GPtrArray *children;
 	int i;
@@ -57,16 +57,13 @@ add_topics_list (EphyNode *topics, EphyNode *bmk,
 	for (l = bmks; l != NULL; l = l->next)
 	{
 		const char *name;
-		xmlChar *xml;
 		EphyNode *node = l->data;
 		xmlNodePtr item_node;
 
 		name = ephy_node_get_property_string
 			(node, EPHY_NODE_KEYWORD_PROP_NAME);
-		xml = xmlEncodeEntitiesReentrant (doc, name);
 		item_node = xmlNewChild (parent, dc_ns, "subject", NULL);
-		xmlNodeSetContent (item_node, xml);
-		xmlFree (xml);
+		xmlNodeSetContent (item_node, name);
 	}
 
 	g_list_free (bmks);
@@ -76,8 +73,7 @@ void
 ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 			   const char *filename)
 {
-	EphyNode *bmks;
-	EphyNode *topics;
+	EphyNode *bmks, *topics, *smart_bmks;
 	xmlDocPtr doc;
 	xmlNodePtr root, xml_node, channel_node, channel_seq_node;
 	xmlNsPtr ephy_ns, rdf_ns, dc_ns;
@@ -113,6 +109,7 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 
 	bmks = ephy_bookmarks_get_bookmarks (bookmarks);
 	topics = ephy_bookmarks_get_keywords (bookmarks);
+	smart_bmks = ephy_bookmarks_get_smart_bookmarks (bookmarks);
 
 	children = ephy_node_get_children (bmks);
 	for (i = 0; i < children->len; i++)
@@ -120,14 +117,13 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 		EphyNode *kid;
 		const char *url, *title;
 		xmlNodePtr item_node;
-		xmlChar *encoded_link, *encoded_title;
+		xmlChar *encoded_link;
 		char *link = NULL;
 		gboolean smart_url;
 
 		kid = g_ptr_array_index (children, i);
 
-		smart_url = ephy_node_get_property_boolean
-			(kid, EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS);
+		smart_url = ephy_node_has_child (smart_bmks, kid);
 		url = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
 		title = ephy_node_get_property_string
@@ -163,8 +159,7 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 		item_node = xmlNewChild (root, NULL, "item", NULL);
 		xmlSetNsProp (item_node, rdf_ns, "about", link);
 
-		encoded_title = xmlEncodeEntitiesReentrant (doc, title);
-		xml_node = xmlNewChild (item_node, NULL, "title", encoded_title);
+		xml_node = xmlNewChild (item_node, NULL, "title", title);
 
 		xml_node = xmlNewChild (item_node, NULL, "link", encoded_link);
 
@@ -176,10 +171,9 @@ ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 			xmlFree (encoded_url);
 		}
 
-		add_topics_list (topics, kid, item_node, dc_ns, doc);
+		add_topics_list (topics, kid, item_node, dc_ns);
 
 		xmlFree (encoded_link);
-		xmlFree (encoded_title);
 		g_free (link);
 	}
 	ephy_node_thaw (bmks);
