@@ -30,13 +30,9 @@
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <gtk/gtkwindow.h>
-#include <libgnomevfs/gnome-vfs-init.h>
-#include <glade/glade-init.h>
 
 #define EPHY_FACTORY_OAFIID "OAFIID:GNOME_Epiphany_Automation_Factory"
 
-static gboolean
-ephy_main_automation_init (void);
 static gint
 ephy_main_translate_url_arguments (poptContext context, gchar ***urls);
 static gboolean
@@ -51,13 +47,11 @@ static gboolean open_in_new_window    = FALSE;  /* force open in a new window?
 static gboolean open_fullscreen       = FALSE;  /* open ephy in full screen ? */
 static gchar   *session_filename      = NULL;   /* the session filename         */
 static gchar   *bookmark_url          = NULL;   /* the temp bookmark to add     */
+static gchar   *bookmarks_file        = NULL;   /* the bookmarks file to import */
 static gboolean close_option          = FALSE;  /* --close                      */
-static gboolean quit_option           = FALSE;  /* --quit                       */
-static gboolean ephy_server_mode    = FALSE;
 static gboolean open_as_bookmarks_editor = FALSE; /* --bookmarks-editor	*/
 static gboolean open_as_nautilus_view = FALSE;
 
-static BonoboObject *automation_object;
 static gint n_urls;
 static gchar **url;
 static gboolean first_instance;
@@ -85,19 +79,15 @@ static struct poptOption popt_options[] =
 	{ "load-session", 'l', POPT_ARG_STRING, &session_filename, 0,
 	  N_("Load the given session file"),
 	  N_("FILE") },
-	{ "server", 's', POPT_ARG_NONE, &ephy_server_mode, 0,
-	  N_("Don't open any windows; instead act as a server "
-	     "for quick startup of new Epiphany instances"),
-	  NULL },
 	{ "add-bookmark", 't', POPT_ARG_STRING, &bookmark_url,
 	  0, N_("Add a bookmark (don't open any window)"),
 	  N_("URL")},
+	{ "import-bookmarks", '\0', POPT_ARG_STRING, &bookmarks_file,
+	  0, N_("Import bookmarks from the given file"),
+	  N_("FILE") },
 	{ "close", 'c', POPT_ARG_NONE, &close_option, 0,
 	  N_("Close all Epiphany windows"),
 	  NULL },
-	{ "quit", 'q', POPT_ARG_NONE, &quit_option, 0,
-	  N_("Same as --close, but exits server mode too"),
-	  NULL },
 	{ "nautilus-view", 'v', POPT_ARG_NONE, &open_as_nautilus_view, 0,
 	  N_("Used internally by the nautilus view"),
 	  NULL },
@@ -115,7 +105,7 @@ main (int argc, char *argv[])
 	poptContext context;
         GValue context_as_value = { 0 };
 	GnomeProgram *program;
-	char *file;
+	CORBA_Object factory;
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
@@ -141,27 +131,27 @@ main (int argc, char *argv[])
 	/* load arguments that aren't regular options (urls to load) */
         n_urls = ephy_main_translate_url_arguments (context, &url);
 
-	first_instance = ephy_main_automation_init ();
+	g_value_unset (&context_as_value);
 
-	if (first_instance)
+	factory = bonobo_activation_activate_from_id
+		(EPHY_FACTORY_OAFIID,
+		 Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
+		 NULL, NULL);
+
+	if (factory != NULL)
 	{
-		gnome_vfs_init ();
-
-		glade_gnome_init ();
+		first_instance = FALSE;
+		ephy_main_start (NULL);
+	}
+	else
+	{
+		first_instance = TRUE;
 
 		ephy_shell_new ();
 
-		file = gnome_program_locate_file
-			(NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-			 "epiphany.png", TRUE, NULL);
-		gtk_window_set_default_icon_from_file (file, NULL);
-		g_free (file);
-
 		g_idle_add ((GSourceFunc) ephy_main_start, NULL);
 
 		bonobo_main ();
-
-		gnome_vfs_shutdown ();
 	}
 
 	return 0;
@@ -195,12 +185,6 @@ ephy_main_start (gpointer data)
 		gtk_dialog_run (GTK_DIALOG (dialog));
 
 	}
-	/* FIXME ephy --server doesnt work when not first istance */
-	/* Server mode */
-	else if (ephy_server_mode)
-	{
-		g_object_ref (G_OBJECT(ephy_shell));
-	}
 	/* Launch the bookmarks editor */
 	else if (open_as_bookmarks_editor)
 	{
@@ -213,32 +197,25 @@ ephy_main_start (gpointer data)
 		GNOME_EphyAutomation_loadSession
 			(gaserver, session_filename, &corba_env);
 	}
+	/* if we're given a bookmarks file to import... */
+	else if (bookmarks_file != NULL)
+	{
+		GNOME_EphyAutomation_importBookmarks
+			(gaserver, bookmarks_file, &corba_env);
+	}
 	/* if found and we're given a bookmark to add... */
 	else if (bookmark_url != NULL)
 	{
 		GNOME_EphyAutomation_addBookmark
 			(gaserver, bookmark_url, &corba_env);
 	}
-	else if (close_option || quit_option)
+	else if (close_option)
 	{
 		GNOME_EphyAutomation_quit
-			(gaserver, quit_option, &corba_env);
+			(gaserver, &corba_env);
 	}
 	/* provided with urls? */
-	else if (n_urls == 0 &&
-		 !open_as_nautilus_view)
-	{
-		/* no, open a default window */
-		GNOME_EphyAutomation_loadurl
-			(gaserver, "",
-			 open_fullscreen,
-			 open_in_existing,
-			 open_in_new_window,
-			 open_in_new_tab,
-			 !noraise,
-			 &corba_env);
-	}
-	else
+	else if (!open_as_nautilus_view)
 	{
 		/* open all of the urls */
 		for (i = 0; i < n_urls; i++)
@@ -254,8 +231,11 @@ ephy_main_start (gpointer data)
 		}
 	}
 
-	/* Unref so it will exit if no more used */
-	if (first_instance)
+	/* Unref so it will exit if no more used
+	 * If started with --nautilus-view, only exit when the
+	 * last view has been destroyed.
+	 */
+	if (first_instance && !open_as_nautilus_view)
 	{
 		g_object_unref (G_OBJECT(ephy_shell));
 	}
@@ -272,26 +252,16 @@ ephy_main_start (gpointer data)
 	return FALSE;
 }
 
-static gboolean
-ephy_main_automation_init (void)
+/* dummy argument to open a default window */
+static gint
+ephy_main_dummy_url_argument (gchar ***urls)
 {
-	CORBA_Object factory;
+       *urls = g_new0 (gchar *, 2);
 
-	factory = bonobo_activation_activate_from_id
-		(EPHY_FACTORY_OAFIID,
-		 Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
-		 NULL, NULL);
+       (*urls)[0] = g_strdup ("");
+       (*urls)[1] = NULL;
 
-	if (!factory)
-	{
-		automation_object = ephy_automation_new ();
-		return TRUE;
-	}
-	else
-	{
-		ephy_main_start (NULL);
-		return FALSE;
-	}
+       return 1;
 }
 
 /**
@@ -308,8 +278,7 @@ ephy_main_translate_url_arguments (poptContext context, gchar ***urls)
         /* any context remaining? */
         if (context == NULL)
         {
-                *urls = NULL;
-                return 0;
+		return ephy_main_dummy_url_argument (urls);
         }
 
         /* get the args and check */
@@ -317,8 +286,7 @@ ephy_main_translate_url_arguments (poptContext context, gchar ***urls)
         if (args == NULL)
         {
                 poptFreeContext (context);
-                *urls = NULL;
-                return 0;
+		return ephy_main_dummy_url_argument (urls);
         }
 
         /* count args */
