@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.76.2.4 2003/12/22 15:35:33 chpe Exp $
+ *  $Id: window-commands.c,v 1.83 2003/10/16 17:28:46 marco Exp $
  */
 
 #include <config.h>
@@ -30,9 +30,6 @@
 #include "pdm-dialog.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-new-bookmark.h"
-#include "egg-toggle-action.h"
-#include "egg-editable-toolbar.h"
-#include "egg-toolbar-editor.h"
 #include "ephy-file-helpers.h"
 #include "toolbar.h"
 #include "ephy-state.h"
@@ -40,6 +37,8 @@
 #include "ephy-zoom.h"
 #include "prefs-dialog.h"
 #include "ephy-toolbars-model.h"
+#include "egg-editable-toolbar.h"
+#include "egg-toolbar-editor.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -50,6 +49,7 @@
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnome/gnome-program.h>
 #include <gtk/gtkeditable.h>
+#include <gtk/gtktoggleaction.h>
 
 enum
 {
@@ -57,21 +57,21 @@ enum
 };
 
 void
-window_cmd_edit_find (EggAction *action,
+window_cmd_edit_find (GtkAction *action,
 		      EphyWindow *window)
 {
 	ephy_window_find (window);
 }
 
 void
-window_cmd_file_print (EggAction *action,
+window_cmd_file_print (GtkAction *action,
 		       EphyWindow *window)
 {
 	ephy_window_print (window);
 }
 
 void
-window_cmd_go_back (EggAction *action,
+window_cmd_go_back (GtkAction *action,
 		    EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -85,7 +85,7 @@ window_cmd_go_back (EggAction *action,
 }
 
 void
-window_cmd_go_up (EggAction *action,
+window_cmd_go_up (GtkAction *action,
 		  EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -99,7 +99,7 @@ window_cmd_go_up (EggAction *action,
 }
 
 void
-window_cmd_file_send_to	(EggAction *action,
+window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyTab *tab;
@@ -142,7 +142,7 @@ window_cmd_file_send_to	(EggAction *action,
 }
 
 void
-window_cmd_go_forward (EggAction *action,
+window_cmd_go_forward (GtkAction *action,
 		       EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -156,7 +156,7 @@ window_cmd_go_forward (EggAction *action,
 }
 
 void
-window_cmd_go_home (EggAction *action,
+window_cmd_go_home (GtkAction *action,
 		    EphyWindow *window)
 {
 	char *location;
@@ -176,14 +176,14 @@ window_cmd_go_home (EggAction *action,
 }
 
 void
-window_cmd_go_location (EggAction *action,
+window_cmd_go_location (GtkAction *action,
 		        EphyWindow *window)
 {
 	ephy_window_activate_location (window);
 }
 
 void
-window_cmd_view_stop (EggAction *action,
+window_cmd_view_stop (GtkAction *action,
 		      EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -197,7 +197,7 @@ window_cmd_view_stop (EggAction *action,
 }
 
 void
-window_cmd_view_reload (EggAction *action,
+window_cmd_view_reload (GtkAction *action,
 		        EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -238,7 +238,7 @@ window_cmd_view_reload (EggAction *action,
 }
 
 void
-window_cmd_file_new_window (EggAction *action,
+window_cmd_file_new_window (GtkAction *action,
 		            EphyWindow *window)
 {
 	EphyTab *tab;
@@ -252,7 +252,7 @@ window_cmd_file_new_window (EggAction *action,
 }
 
 void
-window_cmd_file_new_tab (EggAction *action,
+window_cmd_file_new_tab (GtkAction *action,
 		         EphyWindow *window)
 {
 	EphyTab *tab;
@@ -267,14 +267,14 @@ window_cmd_file_new_tab (EggAction *action,
 }
 
 void
-window_cmd_go_bookmarks (EggAction *action,
+window_cmd_go_bookmarks (GtkAction *action,
 			 EphyWindow *window)
 {
 	ephy_shell_show_bookmarks_editor (ephy_shell, GTK_WIDGET (window));
 }
 
 void
-window_cmd_file_bookmark_page (EggAction *action,
+window_cmd_file_bookmark_page (GtkAction *action,
 			       EphyWindow *window)
 {
 	EphyTab *tab;
@@ -318,7 +318,7 @@ window_cmd_file_bookmark_page (EggAction *action,
 }
 
 void
-window_cmd_file_open (EggAction *action,
+window_cmd_file_open (GtkAction *action,
 		      EphyWindow *window)
 {
 	char *dir, *ret_dir, *file;
@@ -367,7 +367,7 @@ window_cmd_file_open (EggAction *action,
 }
 
 void
-window_cmd_file_save_as (EggAction *action,
+window_cmd_file_save_as (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -383,15 +383,13 @@ window_cmd_file_save_as (EggAction *action,
 	ephy_embed_utils_save (GTK_WIDGET(window),
 			       _("Save As"),
 			       CONF_STATE_SAVE_DIR,
-			       TRUE,
-			       TRUE,
-			       persist);
+			       TRUE, persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
 
 void
-window_cmd_file_close_window (EggAction *action,
+window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
 {
 	EphyTab *tab;
@@ -403,7 +401,7 @@ window_cmd_file_close_window (EggAction *action,
 }
 
 void
-window_cmd_edit_cut (EggAction *action,
+window_cmd_edit_cut (GtkAction *action,
 		     EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -423,7 +421,7 @@ window_cmd_edit_cut (EggAction *action,
 }
 
 void
-window_cmd_edit_copy (EggAction *action,
+window_cmd_edit_copy (GtkAction *action,
 		      EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -444,7 +442,7 @@ window_cmd_edit_copy (EggAction *action,
 }
 
 void
-window_cmd_edit_paste (EggAction *action,
+window_cmd_edit_paste (GtkAction *action,
 		       EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -465,7 +463,7 @@ window_cmd_edit_paste (EggAction *action,
 }
 
 void
-window_cmd_edit_select_all (EggAction *action,
+window_cmd_edit_select_all (GtkAction *action,
 			    EphyWindow *window)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
@@ -486,7 +484,7 @@ window_cmd_edit_select_all (EggAction *action,
 }
 
 void
-window_cmd_edit_find_next (EggAction *action,
+window_cmd_edit_find_next (GtkAction *action,
 			   EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -498,7 +496,7 @@ window_cmd_edit_find_next (EggAction *action,
 }
 
 void
-window_cmd_edit_find_prev (EggAction *action,
+window_cmd_edit_find_prev (GtkAction *action,
 			   EphyWindow *window)
 {
 	EphyEmbed *embed;
@@ -510,7 +508,7 @@ window_cmd_edit_find_prev (EggAction *action,
 }
 
 void
-window_cmd_view_bookmarks_bar (EggAction *action,
+window_cmd_view_bookmarks_bar (GtkAction *action,
 			       EphyWindow *window)
 {
 	EmbedChromeMask mask;
@@ -518,7 +516,7 @@ window_cmd_view_bookmarks_bar (EggAction *action,
 	gboolean current_state;
 
 	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	current_state = (mask & EMBED_CHROME_BOOKMARKSBARON) > 0;
 
 	if (active != current_state)
@@ -529,7 +527,7 @@ window_cmd_view_bookmarks_bar (EggAction *action,
 }
 
 void
-window_cmd_view_toolbar (EggAction *action,
+window_cmd_view_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
 	EmbedChromeMask mask;
@@ -537,7 +535,7 @@ window_cmd_view_toolbar (EggAction *action,
 	gboolean current_state;
 
 	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	current_state = (mask & EMBED_CHROME_TOOLBARON) > 0;
 
 	if (active != current_state)
@@ -548,7 +546,7 @@ window_cmd_view_toolbar (EggAction *action,
 }
 
 void
-window_cmd_view_statusbar (EggAction *action,
+window_cmd_view_statusbar (GtkAction *action,
 			   EphyWindow *window)
 {
 	EmbedChromeMask mask;
@@ -556,7 +554,7 @@ window_cmd_view_statusbar (EggAction *action,
 	gboolean current_state;
 
 	mask = ephy_window_get_chrome (window);
-	active = EGG_TOGGLE_ACTION (action)->active;
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	current_state = (mask & EMBED_CHROME_STATUSBARON) > 0;
 
 	if (active != current_state)
@@ -567,10 +565,10 @@ window_cmd_view_statusbar (EggAction *action,
 }
 
 void
-window_cmd_view_fullscreen (EggAction *action,
+window_cmd_view_fullscreen (GtkAction *action,
 			    EphyWindow *window)
 {
-	if (EGG_TOGGLE_ACTION (action)->active)
+	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
 	{
 		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
@@ -581,28 +579,28 @@ window_cmd_view_fullscreen (EggAction *action,
 }
 
 void
-window_cmd_view_zoom_in	(EggAction *action,
+window_cmd_view_zoom_in	(GtkAction *action,
 			 EphyWindow *window)
 {
 	ephy_window_set_zoom (window, ZOOM_IN);
 }
 
 void
-window_cmd_view_zoom_out (EggAction *action,
+window_cmd_view_zoom_out (GtkAction *action,
 			  EphyWindow *window)
 {
 	ephy_window_set_zoom (window, ZOOM_OUT);
 }
 
 void
-window_cmd_view_zoom_normal (EggAction *action,
+window_cmd_view_zoom_normal (GtkAction *action,
 			     EphyWindow *window)
 {
 	ephy_window_set_zoom (window, 1.0);
 }
 
 void
-window_cmd_view_page_source (EggAction *action,
+window_cmd_view_page_source (GtkAction *action,
 			     EphyWindow *window)
 {
 	EphyTab *tab;
@@ -616,14 +614,14 @@ window_cmd_view_page_source (EggAction *action,
 }
 
 void
-window_cmd_go_history (EggAction *action,
+window_cmd_go_history (GtkAction *action,
 		       EphyWindow *window)
 {
 	ephy_shell_show_history_window (ephy_shell, GTK_WIDGET (window));
 }
 
 void
-window_cmd_edit_personal_data (EggAction *action,
+window_cmd_edit_personal_data (GtkAction *action,
 		               EphyWindow *window)
 {
 	EphyDialog *dialog;
@@ -634,7 +632,7 @@ window_cmd_edit_personal_data (EggAction *action,
 }
 
 void
-window_cmd_edit_prefs (EggAction *action,
+window_cmd_edit_prefs (GtkAction *action,
 		       EphyWindow *window)
 {
 	EphyDialog *dialog;
@@ -665,7 +663,7 @@ toolbar_editor_response_cb (GtkDialog  *dialog,
 		break;
 	case RESPONSE_ADD_TOOLBAR:
 		model = EPHY_TOOLBARS_MODEL
-			(ephy_shell_get_toolbars_model (ephy_shell));
+			(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 		n = egg_toolbars_model_n_toolbars (EGG_TOOLBARS_MODEL (model));
 		egg_toolbars_model_add_toolbar (EGG_TOOLBARS_MODEL (model),
 						n - 1, "UserCreated");
@@ -677,7 +675,7 @@ toolbar_editor_response_cb (GtkDialog  *dialog,
 }
 
 void
-window_cmd_edit_toolbar (EggAction *action,
+window_cmd_edit_toolbar (GtkAction *action,
 			 EphyWindow *window)
 {
 	GtkWidget *editor;
@@ -686,7 +684,7 @@ window_cmd_edit_toolbar (EggAction *action,
 	GtkWidget *dialog;
 
 	model = EPHY_TOOLBARS_MODEL
-		(ephy_shell_get_toolbars_model (ephy_shell));
+		(ephy_shell_get_toolbars_model (ephy_shell, FALSE));
 	t = ephy_window_get_toolbar (window);
 
 	dialog = gtk_dialog_new ();
@@ -696,7 +694,7 @@ window_cmd_edit_toolbar (EggAction *action,
 				      GTK_WINDOW (window));
 
 	editor = egg_toolbar_editor_new
-		(EGG_MENU_MERGE (window->ui_merge),
+		(GTK_UI_MANAGER (window->ui_merge),
 		 EGG_TOOLBARS_MODEL (model));
 	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
 					 ephy_file ("epiphany-toolbar.xml"));
@@ -731,14 +729,14 @@ window_cmd_edit_toolbar (EggAction *action,
 }
 
 void
-window_cmd_help_contents (EggAction *action,
+window_cmd_help_contents (GtkAction *action,
 			 EphyWindow *window)
 {
 	ephy_gui_help (GTK_WINDOW (window), "epiphany", NULL);
 }
 
 void
-window_cmd_help_about (EggAction *action,
+window_cmd_help_about (GtkAction *action,
 		       GtkWidget *window)
 {
 	static GtkWidget *about = NULL;
@@ -750,9 +748,9 @@ window_cmd_help_about (EggAction *action,
 
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <marco@gnome.org>",
-		"Xan Lopez <xan@gnome.org>",
+		"Xan Lopez <xan@masilla.org>",
 		"David Bordoley <bordoley@msu.edu>",
-		"Christian Persch <chpe@gnome.org>",
+		"Christian Persch <chpe+epiphany@stud.uni-saarland.de>",
 		NULL
 	};
 
@@ -815,7 +813,7 @@ window_cmd_help_about (EggAction *action,
 }
 
 void
-window_cmd_tabs_next (EggAction *action,
+window_cmd_tabs_next (GtkAction *action,
 		      EphyWindow *window)
 {
 	GtkNotebook *nb;
@@ -834,7 +832,7 @@ window_cmd_tabs_next (EggAction *action,
 }
 
 void
-window_cmd_tabs_previous (EggAction *action,
+window_cmd_tabs_previous (GtkAction *action,
 			  EphyWindow *window)
 {
 	GtkNotebook *nb;
@@ -853,7 +851,7 @@ window_cmd_tabs_previous (EggAction *action,
 }
 
 void
-window_cmd_tabs_move_left  (EggAction *action,
+window_cmd_tabs_move_left  (GtkAction *action,
 			    EphyWindow *window)
 {
 	GtkWidget *notebook;
@@ -871,7 +869,7 @@ window_cmd_tabs_move_left  (EggAction *action,
 	}
 }
 
-void window_cmd_tabs_move_right (EggAction *action,
+void window_cmd_tabs_move_right (GtkAction *action,
 				 EphyWindow *window)
 {
 	GtkWidget *notebook;
@@ -892,7 +890,7 @@ void window_cmd_tabs_move_right (EggAction *action,
 }
 
 void
-window_cmd_tabs_detach  (EggAction *action,
+window_cmd_tabs_detach  (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyTab *tab;
@@ -913,11 +911,11 @@ window_cmd_tabs_detach  (EggAction *action,
 }
 
 void
-window_cmd_load_location (EggAction *action,
+window_cmd_load_location (GtkAction *action,
 			  EphyWindow *window)
 {
 	Toolbar *toolbar;
-	char *location;
+	const char *location;
 
 	toolbar = ephy_window_get_toolbar (window);
 	location = toolbar_get_location (toolbar);
@@ -925,7 +923,6 @@ window_cmd_load_location (EggAction *action,
 	if (location)
 	{
 		ephy_window_load_url (window, location);
-		g_free (location);
 	}
 }
 
